/**
 *  Created     :
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */



// ------------------------------------
// GSXR_API_通用基础类
// ------------------------------------
#include "app_api/gsxr.h"
#include "SimpleRuntime.h"
/**
 *  说明：
 *      转换 GSXR_Result 返回码为字符串
 *  参数：
 *       [in] result GSXR_Result 返回码
 *      [out] string 输出转换之字符串
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_API_Unsupported
 *      GSXR_Error_Argument_Invalid
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_通用基础类
 */
GSXR_Result GSXR_ResultToString(
        GSXR_Result     result,
        char            string[GSXR_COMMON_STRING_MAX_SIZE]){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      创建并初始 GSXR Runtime 实例, 始能XR Runtime运行平台
 *  参数：
 *       [in] createInfo 指向 GSXR_RuntimeCreateInfo 之结构体, 内含 Runtime 初始资讯
 *      [out] runtime 指向 GSXR_Runtime 句柄之指针, 实例创建成功输出一有效句柄, 创建失败输出 GSXR_NULL_HANDLE
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_ApplicationType_Unsupported
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Out_Of_Memory
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_通用基础类
 */
GSXR_Result GSXR_CreateRuntime(
        const GSXR_RuntimeCreateInfo*   createInfo,
        GSXR_Runtime*                   runtime){

    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_CreateRuntime(createInfo,runtime);
}
/**
 *  说明：
 *      终止并销毁 GSXR Runtime 实例
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_通用基础类
 */
GSXR_Result GSXR_DestroyRuntime(
        GSXR_Runtime        runtime){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_DestroyRuntime(runtime);
}
/**
 *  说明：
 *      获取 GSXR Runtime 属性
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *      [out] runtimeProperties 指向 GSXR_RuntimeProperties 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_API_Unsupported
 *      GSXR_Error_Handle_Invalid
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_通用基础类
 */
GSXR_Result GSXR_GetRuntimeProperties(
        GSXR_Runtime                runtime,
        GSXR_RuntimeProperties*     runtimeProperties){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      从事件队列中获取 GSXR 事件, 返回 GSXR_Result_Success 代表队列中尚有事件, 需再次调用函数获取下一事件,
 *      直至返回 GSXR_Result_Event_Unavailable 代表队列中已无新事件
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *      [out] event 指向 GSXR_Event 结构体, 输出指针需依事件类型转换至该事件之对应结构体指针
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Result_Event_Unavailable
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_通用基础类
 */
GSXR_Result GSXR_PollEvent(
        GSXR_Runtime        runtime,
        GSXR_Event*         event){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_PollEvent(runtime,event);
}