/**
 *  Created     :
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include "SimpleRuntime.h"
// ------------------------------------
// GSXR_API_设备输入类
// ------------------------------------
#include "app_api/gsxr_device.h"
/**
 *  说明：
 *      获取当前支持的所有设备类型
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *      [out] deviceTypeFlags 指向 GSXR_Flags64 数值之指针, 数值为 GSXR_DeviceType 之位掩码, 输出可支持的所有设备类型
 *
 *  案例：(支持头部控件、左手控制器、右手控制器)
 *      *deviceTypeFlags = GSXRMaskFromId(GSXR_DeviceType_HMD_VR)+GSXRMaskFromId(GSXR_DeviceType_Controller_Left)+GSXRMaskFromId(GSXR_DeviceType_Controller_Right)
 *
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备输入类
 */
GSXR_Result GSXR_GetSupportedDevices(
        GSXR_Runtime        runtime,
        GSXR_Flags64*       deviceTypeFlags){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取当前主手柄，会被用来发射UI交互射线
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *      [out] deviceTypeFlags 指向 GSXR_Flags64 数值之指针, 数值为GSXR_DeviceType, 输出主手柄设备类型
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备输入类
 */
GSXR_Result GSXR_GetPrimaryDeviceType(
        GSXR_Runtime        runtime,
        GSXR_Flags64*       deviceTypeFlags){
    return GSXR_Result::GSXR_Result_Success;
}
/**
*  说明：
*      设置当前主手柄设备类型
*  参数：
*       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
*       [in] deviceType数值为GSXR_DeviceType, 指定主手柄设备类型
*  返回值：
*      GSXR_Result_Success
*      GSXR_Error_Operation_Failed
*      GSXR_Error_Handle_Invalid
*      GSXR_Error_Runtime_Unavailable
*  备注：
*      无
*  所属：
*      GSXR_API_设备输入类
*/
GSXR_Result GSXR_SetPrimaryDeviceType(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取当前设备支持的输入类型
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *      [out] deviceInputTypeFlag 指向 GSXR_Flags64 数值之指针, 数值为 GSXR_Device_InputType类型之一, 输出支持的设备输入类型
 *            如果设备类型是GSXR_DeviceType_Controller_Left，则deviceInputTypeFlag为GSXR_Touch或者是GSXR_Joystick类型之一，即*deviceInputTypeFlag = GSXR_Touch 或者*deviceInputTypeFlag =GSXR_Joystick
 *            如果设备类型是GSXR_DeviceType_Controller_Right，则deviceInputTypeFlag为GSXR_Touch或者是GSXR_Joystick类型之一,即*deviceInputTypeFlag = GSXR_Touch 或者*deviceInputTypeFlag =GSXR_Joystick
 *            如果设备类型是GSXR_DeviceType_GamePad，则deviceInputTypeFlag是按GSXR_Gamepad类型之一,即*deviceInputTypeFlag = GSXR_Gamepad
 *            如果设备类型是GSXR_DeviceType_HMD_VR，则deviceInputTypeFlag是按GSXR_Hmd类型,即*deviceInputTypeFlag = GSXR_Hmd
 *            对于其他设备类型暂未开放输入类型，则deviceInputTypeFlag为GSXR_NO_Supported_InputType
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备输入类
 */
GSXR_Result GSXR_GetSupportedDeviceInputType(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType,
        GSXR_Flags64*       deviceInputTypeFlag){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取设备连线状态
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *      [out] connected 指向 GSXR_Bool32 数值之指针, 数值输出 GSXR_TRUE 表示已连线, GSXR_FALSE 表示未连线
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备输入类
 */
GSXR_Result GSXR_IsDeviceConnected(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType,
        GSXR_Bool32*        connected){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取显示设备标识号
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *      [out] displayId 指向 GSXR_DisplayId 标识号之指针, 若设备具显示屏能力则输出显示标识号, 否则输出 GSXR_NULL_DISPLAY_ID
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备输入类
 */
GSXR_Result GSXR_GetDisplay(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType,
        GSXR_DisplayId*     displayId){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取设备输入部件所具备的输入类型配置
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *            目前开放了GSXR_DeviceType_Controller_Left、GSXR_DeviceType_Controller_Right、GSXR_DeviceType_GamePad、GSXR_DeviceType_HMD_VR等设备具备输入类型（GSXR_Device_InputType）
 *       [in] inputType 指定输入类型,根据GSXR_GetSupportedDeviceInputType获取，并且要求一致
 *            如果设备类型是GSXR_DeviceType_Controller_Left，则inputType为GSXR_Touch或者是GSXR_Joystick类型
 *            如果设备类型是GSXR_DeviceType_Controller_Right，则inputType为GSXR_Touch或者是GSXR_Joystick类型
 *            如果设备类型是GSXR_DeviceType_GamePad，则inputType是按GSXR_Gamepad类型
 *            如果设备类型是GSXR_DeviceType_HMD_VR，则inputType是按GSXR_Hmd类型
 *            此参数必须与GSXR_GetSupportedDeviceInputType保持一致
 *       [in] inputId 指定输入部件Id
 *            如果设备类型是GSXR_DeviceType_Controller_Left，则inputId是按GSXR_Touch或者是GSXR_Joystick类型下InputId
 *            如果设备类型是GSXR_DeviceType_Controller_Right，则inputId是按GSXR_Touch或者是GSXR_Joystick类型下InputId
 *            如果设备类型是GSXR_DeviceType_GamePad，则inputId是按GSXR_Gamepad类型下InputId
 *            如果设备类型是GSXR_DeviceType_HMD_VR，则inputId是按GSXR_Hmd类型下InputId
 *            上层根据inputType，进行传递inputId，并转成int类型
 *      [out] configuration 指向 GSXR_Flags64 数值之指针, 数值为 GSXR_InputId_InputType 之位掩码, 输出此输入部件所具备的输入类型配置
 *      案例：(表示摇杆类型下的trigger按键具备按键、触摸、轴值类型)
 *          deviceType：GSXR_DeviceType_Controller_Left
 *          inputType：GSXR_Joystick
 *          inputId: GSXR_Joystick_InputId_Trigger
 *          *configuration = GSXR_InputId_InputType_Click|GSXR_InputId_InputType_Touch|GSXR_InputId_InputType_Analog_1D
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备输入类
 */
GSXR_Result GSXR_GetInputIdConfiguration(
        GSXR_Runtime                runtime,
        GSXR_DeviceType             deviceType,
        GSXR_Device_InputType       inputType,
        int                         inputId,
        GSXR_Flags64*               configuration){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取设备输入部件之点击状态
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *       [in] inputType 指定输入类型,根据GSXR_GetSupportedDeviceInputType获取，并且一致
 *            如果设备类型是GSXR_DeviceType_Controller_Left，则inputType为GSXR_Touch或者是GSXR_Joystick类型
 *            如果设备类型是GSXR_DeviceType_Controller_Right，则inputType为GSXR_Touch或者是GSXR_Joystick类型
 *            如果设备类型是GSXR_DeviceType_GamePad，则inputType是按GSXR_Gamepad类型
 *            如果设备类型是GSXR_DeviceType_HMD_VR，则inputType是按GSXR_Hmd类型
 *            此参数必须与GSXR_GetSupportedDeviceInputType保持一致
 *      [out] clicks 指向GSXR_Flags64数值之指针，数值按输入类型下的输入ID为位掩码储存InputId的点击状态信息，
 *            如果设备类型是GSXR_DeviceType_Controller_Left，则clicks是按inputType类型下InputId为位掩码来存储点击状态信息
 *            如果设备类型是GSXR_DeviceType_Controller_Right，则clicks是按inputType类型下InputId为位掩码来存储点击状态信息
 *            如果设备类型是GSXR_DeviceType_GamePad，则clicks是按inputType类型下InputId为位掩码来存储按键信息
 *            如果设备类型是GSXR_DeviceType_HMD_VR，则clicks是按inputType类型下InputId为位掩码来存储按键信息
 *            如果设备类型是其他设备，暂时未开放GSXR_Device_InputType定义，暂不支持
 *      案例：(X\Y按键按下)
 *          deviceType：GSXR_DeviceType_Controller_Left
 *          inputType：GSXR_Joystick
 *          *clicks = GSXRMaskFromId(GSXR_Joystick_InputId_X)+GSXRMaskFromId(GSXR_Gamepad_InputId_Y)
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  所属：
 *      GSXR_API_设备输入类
 */
GSXR_Result GSXR_GetInputClickStates(
        GSXR_Runtime                runtime,
        GSXR_DeviceType             deviceType,
        GSXR_Device_InputType       inputType,
        GSXR_Flags64*               clicks){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取设备输入部件之触摸状态
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *       [in] inputType 指定输入类型,根据GSXR_GetSupportedDeviceInputType获取，并且一致
 *            如果设备类型是GSXR_DeviceType_Controller_Left，则inputType为GSXR_Touch或者是GSXR_Joystick类型
 *            如果设备类型是GSXR_DeviceType_Controller_Right，则inputType为GSXR_Touch或者是GSXR_Joystick类型
 *            如果设备类型是GSXR_DeviceType_GamePad，则inputType是按GSXR_Gamepad类型
 *            如果设备类型是GSXR_DeviceType_HMD_VR，则inputType是按GSXR_Hmd类型
 *            此参数必须与GSXR_GetSupportedDeviceInputType保持一致
 *       [in] inputId 指定输入部件
 *      [out] touchs 指向 GSXR_Flags64 数值的指针, 数值按输入类型下的输入ID为位掩码储存InputId的触摸状态信息，
 *           如果设备类型是GSXR_DeviceType_Controller_Left，则touchs是按InputType类型下的InputId为位掩码来存储触摸状态信息
 *           如果设备类型是GSXR_DeviceType_Controller_Right，则touchs是按InputType类型下的InputId为位掩码来存储触摸状态信息
 *           如果设备类型是GSXR_DeviceType_Gamepad，则touchs是按InputType类型下的InputId为位掩码来存储触摸状态信息
 *           如果设备类型是GSXR_DeviceType_HMD_VR，则touchs是按inputType类型下InputId为位掩码来存储按键信息
 *           如果设备类型是其他的设备，暂时未开发GSXR_Device_InputType定义，暂不支持
 *      案例：(X\Y按键触摸)
 *          deviceType：GSXR_DeviceType_Controller_Left
 *          inputType：GSXR_Joystick
 *          *touchs = GSXRMaskFromId(GSXR_Joystick_InputId_X)+GSXRMaskFromId(GSXR_Gamepad_InputId_Y)
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  所属：
 *      GSXR_API_设备输入类
 */
GSXR_Result GSXR_GetInputTouchStates(
        GSXR_Runtime                runtime,
        GSXR_DeviceType             deviceType,
        GSXR_Device_InputType       inputType,
        GSXR_Flags64*               touchs){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取设备输入部件之类比数据
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *       [in] inputType 指定输入类型,根据GSXR_GetSupportedDeviceInputType获取，并且一致
 *            如果设备类型是GSXR_DeviceType_Controller_Left，则inputType为GSXR_Touch或者是GSXR_Joystick类型
 *            如果设备类型是GSXR_DeviceType_Controller_Right，则inputType为GSXR_Touch或者是GSXR_Joystick类型
 *            如果设备类型是GSXR_DeviceType_GamePad，则inputType是按GSXR_Gamepad类型
 *            如果设备类型是GSXR_DeviceType_HMD_VR，则inputType是按GSXR_Hmd类型
 *            此参数必须与GSXR_GetSupportedDeviceInputType保持一致
 *       [in] inputId 指定输入部件
 *            如果设备类型是GSXR_DeviceType_Controller_Left，则inputId是按GSXR_Touch或者是GSXR_Joystick类型下InputId
 *            如果设备类型是GSXR_DeviceType_Controller_Right，则inputId是按GSXR_Touch或者是GSXR_Joystick类型下InputId
 *            如果设备类型是GSXR_DeviceType_GamePad，则inputId是按GSXR_Gamepad类型下InputId
 *            如果设备类型是GSXR_DeviceType_HMD_VR，则inputId是按GSXR_Hmd类型下InputId
 *            上层根据inputType，进行传递inputId，并转成int类型
 *      [out] analog 指向二维矢量 GSXR_Vector2f 结构体之指针, 数据范围见备注说明
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      2. analog 数据范围
 *          a. 若输入部件之输入类型为 GSXR_InputId_InputType_Analog_1D, analog 数据范围为 0<=x<=1, y=0
 *          b. 若输入部件之输入类型为 GSXR_InputId_InputType_Analog_2D, analog 数据范围为 -1<=x<=1, -1<=y<=1
 *  所属：
 *      GSXR_API_设备输入类
 */
GSXR_Result GSXR_GetInputAnalogState(
        GSXR_Runtime                runtime,
        GSXR_DeviceType             deviceType,
        GSXR_Device_InputType       inputType,
        int                         inputId,
        GSXR_Vector2f*              analog){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetInputAnalogState(runtime,deviceType,inputType,inputId,analog);
}
/**
 *  说明：
 *      触发设备振动反馈, 可自行设置时长,次数,强度, 亦可交由系统触发一个适合当前设备强度适中的短振动
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *       [in] manual 指定手动或自动调控, GSXR_TRUE 表示 XR 应用自行调控振动反馈, GSXR_FALSE 表示交由系统触发一个适合当前设备强度适中的短振动
 *       [in] haptics 指向 GSXR_HapticVibration 结构体之指针, manual 为 GSXR_FALSE 时可输入 nullptr
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备输出类
 */
GSXR_Result GSXR_TriggerVibration(
        GSXR_Runtime                    runtime,
        GSXR_DeviceType                 deviceType,
        GSXR_Bool32                     manual,
        const GSXR_HapticVibration*     haptics){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取设备的空间自由度跟踪能力
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *      [out] dof 指向 GSXR_DegreeOfFreedom 数值的指针, 输出空间自由度跟踪能力
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备空间类
 */
GSXR_Result GSXR_GetDegreeOfFreedom(
        GSXR_Runtime            runtime,
        GSXR_DeviceType         deviceType,
        GSXR_DegreeOfFreedom*   dof){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      设置 XR 应用之空间原点及偏移量
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] space 指定空间原点类型
 *       [in] offset 指定空间原点偏移量, 若 XR 应用无需自设偏移量可输入 nullptr
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备空间类
 */
GSXR_Result GSXR_SetSpaceOrigin(
        GSXR_Runtime        runtime,
        GSXR_SpaceOrigin    space,
        const GSXR_Posef*   offset){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      重置 XR 应用之位姿或yaw朝向
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] Yaw 指定是否只重置应用Yaw轴朝向或者应用位姿
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备空间类
 */
GSXR_Result GSXR_ResetOrientationAndPosition(
        GSXR_Runtime        runtime,
        float               Yaw){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取设备于指定时间之位姿及速度数据
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *       [in] specifiedTime 指定时间
 *      [out] poseState 指向 GSXR_PoseState 结构体之指针
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备空间类
 */
GSXR_Result GSXR_GetPoseState(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType,
        GSXR_Time           specifiedTime,
        GSXR_PoseState*     poseState){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取空间边界配置资讯
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *      [out] boundsConfiguration 指向 GSXR_SpaceBoundsConfiguration 结构体之指针, 输出边界配置资讯
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备空间类
 */
GSXR_Result GSXR_GetSpaceBoundsConfiguration(
        GSXR_Runtime                    runtime,
        GSXR_SpaceBoundsConfiguration*  boundsConfiguration){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取矩形边界资讯
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *      [out] width 输出矩形边界宽度, 单位为米
 *      [out] length 输出矩形边界长度, 单位为米
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备空间类
 */
GSXR_Result GSXR_GetSpaceBoundsRectangle(
        GSXR_Runtime                    runtime,
        float*                          width,
        float*                          length){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取圆形边界资讯
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *      [out] diameter 输出圆形边界直径, 单位为米
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备空间类
 */
GSXR_Result GSXR_GetSpaceBoundsCircle(
        GSXR_Runtime                    runtime,
        float*                          diameter){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取不规则形边界资讯
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] pointCount 边界点集合数组个数, 需相等于 GSXR_SpaceBoundsConfiguration 的 pointCount 数
 *      [out] pointArray 指向 GSXR_Posef 结构体之数组, 输出边界点集合位姿
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备空间类
 */
GSXR_Result GSXR_GetSpaceBoundsIrregular(
        GSXR_Runtime                    runtime,
        uint32_t                        pointCount,
        GSXR_Posef*                     pointArray){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      设置边界防护墙可见模式
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] visible 指定边界防护墙可见模式
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备空间类
 */
GSXR_Result GSXR_SetSpaceBoundsVisible(
        GSXR_Runtime                    runtime,
        GSXR_SpaceBoundsVisible         visible){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取设备属性
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *      [out] deviceProperties 指向 GSXR_DeviceProperties 结构体之指针
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备信息类
 */
GSXR_Result GSXR_GetDeviceProperties(
        GSXR_Runtime            runtime,
        GSXR_DeviceType         deviceType,
        GSXR_DeviceProperties*  deviceProperties){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取设备电池状态
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] deviceType 指定设备类型
 *      [out] batteryStatus 指向 GSXR_BatteryStatus 结构体之指针
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Device_Disconnected
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_设备信息类
 */
GSXR_Result GSXR_GetBatteryStatus(
        GSXR_Runtime            runtime,
        GSXR_DeviceType         deviceType,
        GSXR_BatteryStatus*     batteryStatus){
    return GSXR_Result::GSXR_Result_Success;
}