/**
 *  Created     :
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include "SimpleRuntime.h"
// ------------------------------------
// GSXR_API 眼球追踪
// ------------------------------------
#include "app_api/gsxr_eye_tracking.h"
/**
 *  说明：
 *      获取支持的眼球数据类型
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *      [out] factors GSXR_EyeFactor 之位掩码, 输出所有支持的眼球数据类型
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_眼球追踪类
 */
GSXR_Result GSXR_GetSupportedEyeFactors(
        GSXR_XrFeature      feature,
        GSXR_Flags64*       factors){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      设置 XR 应用所有需要使用的眼球数据类型
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] request GSXR_EyeFactor 之位掩码, 指定欲使用的眼球数据类型
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      XR 应用需优先调用 GSXR_SetEyeFactorRequest 完成眼球数据请求设置后, 始能获取眼球数据
 *  所属：
 *      GSXR_API_眼球追踪类
 */
GSXR_Result GSXR_SetEyeFactorRequest(
        GSXR_XrFeature      feature,
        GSXR_Flags64        request){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取指定时间之眼球追踪数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] specifiedTime 指定时间
 *      [out] data 指向 GSXR_EyeTrackingData 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_眼球追踪类
 */
GSXR_Result GSXR_GetEyeTrackingData(
        GSXR_XrFeature              feature,
        GSXR_Time                   specifiedTime,
        GSXR_EyeTrackingData*       data){
    return GSXR_Result::GSXR_Result_Success;
}