/**
 *  Created     :
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include "SimpleRuntime.h"
// ------------------------------------
// GSXR_API_脸部识别类
// ------------------------------------
#include "app_api/gsxr_face_tracker.h"
/**
 *  说明：
 *      获得脸部识别结果数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] acquireInfo 指向 GSXR_FaceTrackerAcquireInfo 结构体, 指定脸部识别请求资讯
 *      [out] faceTracker 指向 GSXR_FaceTrackerData 指针
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Out_Of_Memory
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      faceTracker 指针指向的 GSXR_FaceTrackerData 指针指向脸部识别特性的 GSXR_FaceTrackerData 结构,
 *      应用程序用毕脸部识别结果数据必须调用 GSXR_ReleaseFaceTrackerData 释放内存
 *  所属：
 *      GSXR_API_脸部识别类
 */
GSXR_Result GSXR_AcquireFaceTrackerData(
        GSXR_XrFeature                          feature,
        const GSXR_FaceTrackerAcquireInfo*      acquireInfo,
        GSXR_FaceTrackerData**                  faceTracker){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      释放脸部识别结果数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *      [out] faceTracker GSXR_FaceTrackerData 指针, 由 GSXR_AcquireFaceTrackerData 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_脸部识别类
 */
GSXR_Result GSXR_ReleaseFaceTrackerData(
        GSXR_XrFeature              feature,
        GSXR_FaceTrackerData*       faceTracker){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获得脸部比对结果数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] acquireInfo 指向 GSXR_FaceMatchTrackerAcquireInfo 结构体, 指定脸部比对请求资讯
 *      [out] faceMatchTracker 指向 GSXR_FaceMatchTrackerData 指针
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Out_Of_Memory
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      faceMatchTracker 指针指向的 GSXR_FaceMatchTrackerData 指针指向脸部识别特性的 GSXR_FaceMatchTrackerData 结构,
 *      应用程序用毕脸部比对结果数据必须调用 GSXR_ReleaseFaceMatchTrackerData 释放内存
 *  所属：
 *      GSXR_API_脸部识别类
 */
GSXR_Result GSXR_AcquireFaceMatchTrackerData(
        GSXR_XrFeature                              feature,
        const GSXR_FaceMatchTrackerAcquireInfo*     acquireInfo,
        GSXR_FaceMatchTrackerData**                 faceMatchTracker){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      释放脸部比对结果数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *      [out] faceMatchTracker GSXR_FaceMatchTrackerData 指针, 由 GSXR_AcquireFaceMatchTrackerData 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_脸部识别类
 */
GSXR_Result GSXR_ReleaseFaceMatchTrackerData(
        GSXR_XrFeature                  feature,
        GSXR_FaceMatchTrackerData*      faceMatchTracker){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取脸部主特征点数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] facelId 指定脸部标识号
 *      [out] data 指向 GSXR_FacialMainFeatureData 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      GSXR_FacialPortionData 中的特征点数组内存由应用程序根据特征点数目配置
 *  所属：
 *      GSXR_API_脸部识别类
 */
GSXR_Result GSXR_GetFacialMainFeatureData(
        GSXR_XrFeature                  feature,
        GSXR_FaceId                     facelId,
        GSXR_FacialMainFeatureData*     data){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取脸部详细特征点数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] facelId 指定脸部标识号
 *      [out] data 指向 GSXR_FacialDetailFeatureData 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      GSXR_FacialPortionData 中的特征点数组内存由应用程序根据特征点数目配置
 *  所属：
 *      GSXR_API_脸部识别类
 */
GSXR_Result GSXR_GetFacialDetailFeatureData(
        GSXR_XrFeature                  feature,
        GSXR_FaceId                     facelId,
        GSXR_FacialDetailFeatureData*   data){
    return GSXR_Result::GSXR_Result_Success;
}