/**
 *  Created     :
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include "SimpleRuntime.h"
// ------------------------------------
// GSXR_API_手势识别类
// ------------------------------------
#include "app_api/gsxr_hand_tracking.h"
/**
 *  说明：
 *      获取可识别的手势类型
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *      [out] gestures GSXR_HandGestureType 之位掩码, 输出所有支持的手势类型
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_手势识别类
 */
GSXR_Result GSXR_GetSupportedHandGestures(
        GSXR_XrFeature      feature,
        GSXR_Flags64*       gestures){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      设置 XR 应用所有需要使用的手势类型
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] request GSXR_HandGestureType 之位掩码, 指定欲使用的手势类型
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      XR 应用需优先调用 GSXR_SetHandGestureRequest 完成手势请求设置后, 始能获取手势数据
 *  所属：
 *      GSXR_API_手势识别类
 */
GSXR_Result GSXR_SetHandGestureRequest(
        GSXR_XrFeature      feature,
        GSXR_Flags64        request){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取手势数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *      [out] data 指向 GSXR_HandGestureData 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      若识别为双手手势, left 与 right 指定同一双手手势
 *  所属：
 *      GSXR_API_手势识别类
 */
GSXR_Result GSXR_GetHandGestureData(
        GSXR_XrFeature              feature,
        GSXR_HandGestureData*       data){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      注册手势识别事件回调函数
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] callback 手势识别事件回调函数
 *       [in] cookie 函数回调时带回的预备数值
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_手势识别类
 */
GSXR_Result GSXR_RegisterHandGestureEventCallback(
        GSXR_XrFeature                  feature,
        GSXR_HandGestureEventCallback   callback,
        void*                           cookie){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取可被追踪的手部节点
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *      [out] joints (1 << GSXR_HandJoint) 之位掩码, 输出算法支持追踪的手部节点
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_手部追踪类
 */
GSXR_Result GSXR_GetTrackedHandJoints(
        GSXR_XrFeature      feature,
        GSXR_Flags64*       joints){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取手部节点空间追踪能力
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *      [out] validity GSXR_HandJointValidity 之位掩码, 输出手部节点的追踪能力
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_手部追踪类
 */
GSXR_Result GSXR_GetHandJointValidity(
        GSXR_XrFeature      feature,
        GSXR_Flags64*       validity){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取指定时间之手部追踪数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] specifiedTime 指定时间
 *      [out] data 指向 GSXR_HandTrackingData 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_手部追踪类
 */
GSXR_Result GSXR_GetHandTrackingData(
        GSXR_XrFeature              feature,
        GSXR_Time                   specifiedTime,
        GSXR_HandTrackingData*      data){
    return GSXR_Result::GSXR_Result_Success;
}
