/**
 *  Created     :
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include "SimpleRuntime.h"
// ------------------------------------
// GSXR_API_图像识别类
// ------------------------------------
#include "app_api/gsxr_image_tracker.h"
/**
 *  说明：
 *      加载欲识别的模型图像
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] imageInfo 指向 GSXR_ImageInfo 结构体, 指定模型图像资讯
 *      [out] modelId 若模型图像加载成功, 输出模型图像标识号
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Out_Of_Memory
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_图像识别类
 */
GSXR_Result GSXR_LoadModelImage(
        GSXR_XrFeature          feature,
        const GSXR_ImageInfo*   imageInfo,
        GSXR_ImageId*           modelId){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      卸载模型图像
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] modelId GSXR_ImageId 标识号, 由 GSXR_LoadModelImage 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_图像识别类
 */
GSXR_Result GSXR_UnloadModelImage(
        GSXR_XrFeature      feature,
        GSXR_ImageId        modelId){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获得图像识别结果数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *       [in] acquireInfo 指向 GSXR_ImageTrackerAcquireInfo 结构体, 指定图像识别请求资讯
 *      [out] imageTracker 指向 GSXR_ImageTrackerData 指针
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Out_Of_Memory
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      imageTracker 指针指向的 GSXR_ImageTrackerData 指针指向图像识别特性的 GSXR_ImageTrackerData 结构,
 *      应用程序用毕图像识别结果数据必须调用 GSXR_ReleaseImageTrackerData 释放内存
 *  所属：
 *      GSXR_API_图像识别类
 */
GSXR_Result GSXR_AcquireImageTrackerData(
        GSXR_XrFeature                          feature,
        const GSXR_ImageTrackerAcquireInfo*     acquireInfo,
        GSXR_ImageTrackerData**                 imageTracker){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      释放图像识别结果数据
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *      [out] imageTracker GSXR_ImageTrackerData 指针, 由 GSXR_AcquireImageTrackerData 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_图像识别类
 */
GSXR_Result GSXR_ReleaseImageTrackerData(
        GSXR_XrFeature              feature,
        GSXR_ImageTrackerData*      imageTracker){
    return GSXR_Result::GSXR_Result_Success;
}