/**
 *  Created     :
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include "SimpleRuntime.h"
// ------------------------------------
// GSXR_API_渲染视图类
// ------------------------------------
#include "app_api/gsxr_renderer.h"
/**
 *  说明：
 *      创建渲染器
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] createInfo 指向 GSXR_RendererCreateInfo 结构体, 内含渲染器初始资讯
 *      [out] renderer 指向 GSXR_Renderer 句柄之指针, 渲染器创建成功输出一有效句柄, 创建失败输出 GSXR_NULL_HANDLE
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Out_Of_Memory
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染视图类
 */
GSXR_Result GSXR_CreateRenderer(
        GSXR_Runtime                        runtime,
        const GSXR_RendererCreateInfo*      createInfo,
        GSXR_Renderer*                      renderer){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_CreateRenderer(runtime,createInfo,renderer);
}
/**
 *  说明：
 *      销毁渲染器
 *  参数：
 *       [in] renderer GSXR_Renderer 句柄, 由 GSXR_CreateRenderer 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染视图类
 */
GSXR_Result GSXR_DestroyRenderer(
        GSXR_Renderer       renderer){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_DestroyRenderer(renderer);
}
/**
 *  说明：
 *      获取渲染器的视图集数目
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *      [out] count 视图集数目
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染视图类
 */
GSXR_Result GSXR_GetViewSetCount(
        GSXR_Runtime        runtime,
        uint32_t*           count){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetViewSetCount(runtime,count);
}
/**
 *  说明：
 *      获取渲染器的视图集配置
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] count configurations 数组个数, 需相等于 GSXR_GetViewSetCount 的 count 数
 *      [out] configurations 指向 GSXR_ViewSetConfiguration 结构体之数组
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染视图类
 */
GSXR_Result GSXR_GetViewSetConfigurations(
        GSXR_Runtime                    runtime,
        uint32_t                        count,
        GSXR_ViewSetConfiguration*      configurations){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetViewSetConfigurations(runtime,count,configurations);
}
/**
 *  说明：
 *      获取纹理队列功能选项, 包含建议的及所有的支持选项
 *  参数：
 *       [in] renderer GSXR_Renderer 句柄, 由 GSXR_CreateRenderer 获取
 *      [out] recommendedOptions GSXR_TextureQueueOptions 之位掩码, 输出系統建議的纹理队列功能选项
 *      [out] supportedOptions GSXR_TextureQueueOptions 之位掩码, 输出可支持的所有纹理队列功能选项
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染纹理类
 */
GSXR_Result GSXR_GetTextureQueueOptions(
        GSXR_Renderer                   renderer,
        GSXR_Flags64*                   recommendedOptions,
        GSXR_Flags64*                   supportedOptions){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetTextureQueueOptions(renderer,recommendedOptions,supportedOptions);
}
/**
 *  说明：
 *      获取纹理队列支持的纹理格式
 *  参数：
 *       [in] renderer GSXR_Renderer 句柄, 由 GSXR_CreateRenderer 获取
 *       [in] formatCount formats 数组个数, 需相等于 GSXR_ViewSetConfiguration 的 textureFormatCount 数
 *      [out] formats 指向 GSXR_TextureFormat 之数组
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染纹理类
 */
GSXR_Result GSXR_GetSupportedTextureFormats(
        GSXR_Renderer           renderer,
        uint32_t                formatCount,
        GSXR_TextureFormat*     formats){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetSupportedTextureFormats(renderer,formatCount,formats);
}
/**
 *  说明：
 *      创建纹理队列
 *  参数：
 *       [in] renderer GSXR_Renderer 句柄, 由 GSXR_CreateRenderer 获取
 *       [in] createInfo  指向 GSXR_TextureQueueCreateInfo 结构体
 *      [out] textureQueue 纹理队列创建成功输出一有效句柄, 创建失败输出 GSXR_NULL_HANDLE
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染纹理类
 */
GSXR_Result GSXR_CreateTextureQueue(
        GSXR_Renderer                           renderer,
        const GSXR_TextureQueueCreateInfo*      createInfo,
        GSXR_TextureQueue*                      textureQueue){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_CreateTextureQueue(renderer,createInfo,textureQueue);
}
/**
 *  说明：
 *      销毁纹理队列
 *  参数：
 *       [in] textureQueue GSXR_TextureQueue 句柄, 由 GSXR_CreateTextureQueue 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染纹理类
 */
GSXR_Result GSXR_DestroyTextureQueue(
        GSXR_TextureQueue   textureQueue){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_DestroyTextureQueue(textureQueue);
}
/**
 *  说明：
 *      获取纹理队列中的纹理数目
 *  参数：
 *       [in] textureQueue GSXR_TextureQueue 句柄, 由 GSXR_CreateTextureQueue 获取
 *      [out] count 纹理数目
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染纹理类
 */
GSXR_Result GSXR_GetTextureCount(
        GSXR_TextureQueue       textureQueue,
        uint32_t*               count){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetTextureCount(textureQueue,count);
}
/**
 *  说明：
 *      等待纹理队列获得可用纹理
 *  参数：
 *       [in] textureQueue GSXR_TextureQueue 句柄, 由 GSXR_CreateTextureQueue 获取
 *       [in] timeout 持续等待的超时时限值, 单位为 naroseconds
 *      [out] textureElement 指向 GSXR_TextureElement 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Result_Timeout_Expired
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染纹理类
 */
GSXR_Result GSXR_WaitAndAcquireAvailableTexture(
        GSXR_TextureQueue       textureQueue,
        GSXR_Duration           timeout,
        GSXR_TextureElement*    textureElement){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_WaitAndAcquireAvailableTexture(textureQueue,timeout,textureElement);
}
/**
 *  说明：
 *      将纹理释放回纹理队列
 *  参数：
 *       [in] textureQueue GSXR_TextureQueue 句柄, 由 GSXR_CreateTextureQueue 获取
 *       [in] textureElement 指定释放的纹理元素, 由 GSXR_WaitAndAcquireAvailableTexture 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染纹理类
 */
GSXR_Result GSXR_ReleaseTexture(
        GSXR_TextureQueue               textureQueue,
        const GSXR_TextureElement*      textureElement){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_ReleaseTexture(textureQueue,textureElement);
}
/**
 *  说明：
 *      开始渲染循环
 *  参数：
 *       [in] renderer GSXR_Renderer 句柄, 由 GSXR_CreateRenderer 获取
 *       [in] startInfo 指向 GSXR_RenderLooperStartInfo 结构体, 内含渲染循环开始资讯
 *      [out] looper 渲染循环创建成功输出一有效句柄, 创建失败输出 GSXR_NULL_HANDLE
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Out_Of_Memory
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      渲染器状态需处于 GSXR_EventType_RendererState_Available 方能开始渲染循环
 *  所属：
 *      GSXR_API_渲染视图类
 */
GSXR_Result GSXR_StartRenderLooper(
        GSXR_Renderer                           renderer,
        const GSXR_RenderLooperStartInfo*       startInfo,
        GSXR_RenderLooper*                      looper){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_StartRenderLooper(renderer,startInfo,looper);
}
/**
 *  说明：
 *      停止渲染循环
 *  参数：
 *       [in] looper GSXR_RenderLooper 句柄, 由 GSXR_StartRenderLooper 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染视图类
 */
GSXR_Result GSXR_StopRenderLooper(
        GSXR_RenderLooper       looper){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_StopRenderLooper(looper);
}
/**
 *  说明：
 *      设置 XR 应用渲染循环线程 id, Runtime 赋予此线程较高的线程优先权
 *  参数：
 *       [in] looper GSXR_RenderLooper 句柄, 由 GSXR_StartRenderLooper 获取
 *       [in] threadId 应用渲染循环线程 id
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染循环类
 */
GSXR_Result GSXR_SetRenderLooperThread(
        GSXR_RenderLooper       looper,
        uint32_t                threadId){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_SetRenderLooperThread(looper,threadId);
}
/**
 *  说明：
 *      等待调节帧渲染时机，并输出渲染幀的预测显示时间， XR 应用需依据此时间获取对应姿态进行渲染
 *  参数：
 *       [in] looper GSXR_RenderLooper 句柄, 由 GSXR_StartRenderLooper 获取
 *      [out] predictedDisplayTime 输出应用渲染此幀后的显示时间
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染循环类
 */
GSXR_Result GSXR_WaitFrame(
        GSXR_RenderLooper       looper,
        GSXR_Time*              predictedDisplayTime){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_WaitFrame(looper,predictedDisplayTime);
}
/**
 *  说明：
 *      获取视图于指定时间之姿态
 *  参数：
 *       [in] looper GSXR_RenderLooper 句柄, 由 GSXR_StartRenderLooper 获取
 *       [in] specifiedTime 指定时间
 *       [in] viewCount viewPoseState 数组个数, 需相等于 GSXR_ViewSetConfiguration 的 viewCount 数
 *      [out] viewPoseState 指向 GSXR_ViewPoseState 结构体之数组
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染循环类
 */
GSXR_Result GSXR_GetViewPoseState(
        GSXR_RenderLooper       looper,
        GSXR_Time               specifiedTime,
        uint32_t                viewCount,
        GSXR_ViewPoseState*     viewPoseState){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetViewPoseState(looper,specifiedTime,viewCount,viewPoseState);
}
/**
 *  说明：
 *      提交应用的渲染帧资讯给 Runtime 渲染管线进行后续处理
 *  参数：
 *       [in] looper GSXR_RenderLooper 句柄, 由 GSXR_StartRenderLooper 获取
 *       [in] submitInfo  指向 GSXR_FrameSubmitInfo 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染循环类
 */
GSXR_Result GSXR_SubmitFrame(
        GSXR_RenderLooper               looper,
        const GSXR_FrameSubmitInfo*     submitInfo){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_SubmitFrame(looper,submitInfo);
}
/**
 *  说明：
 *      获取渲染功能
 *  参数：
 *       [in] looper GSXR_RenderLooper 句柄, 由 GSXR_StartRenderLooper 获取
 *      [out] recommendedFunctions GSXR_RenderFunctionType 之位掩码, 输出系統建議的渲染功能
 *      [out] supportedFunctions GSXR_RenderFunctionType 之位掩码, 输出可支持的所有渲染功能
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染循环类
 */
GSXR_Result GSXR_GetRenderFunctions(
        GSXR_RenderLooper               looper,
        GSXR_Flags64*                   recommendedFunctions,
        GSXR_Flags64*                   supportedFunctions){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetRenderFunctions(looper,recommendedFunctions,supportedFunctions);
}
/**
 *  说明：
 *      于应用进行渲染前, 预设置渲染功能参数予应用的渲染纹理, 通常用于渲染循环中启用渲染功能
 *  参数：
 *       [in] looper GSXR_RenderLooper 句柄, 由 GSXR_StartRenderLooper 获取
 *       [in] preRenderInfo  指向 GSXR_FrameRenderInfo 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      必须在应用进行渲染前调用
 *  所属：
 *      GSXR_API_渲染循环类
 */
GSXR_Result GSXR_PreRenderFrame(
        GSXR_RenderLooper               looper,
        const GSXR_FrameRenderInfo*     preRenderInfo){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_PreRenderFrame(looper,preRenderInfo);
}
/**
 *  说明：
 *      于应用进行渲染后, 后设置渲染功能参数予应用的渲染纹理, 通常用于渲染循环中停用渲染功能
 *  参数：
 *       [in] looper GSXR_RenderLooper 句柄, 由 GSXR_StartRenderLooper 获取
 *       [in] postRenderInfo  指向 GSXR_FrameRenderInfo 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Call_Flow_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      必须在应用进行渲染后调用
 *  所属：
 *      GSXR_API_渲染循环类
 */
GSXR_Result GSXR_PostRenderFrame(
        GSXR_RenderLooper               looper,
        const GSXR_FrameRenderInfo*     postRenderInfo){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_PostRenderFrame(looper,postRenderInfo);
}
/**
 *  说明：
 *      获取现存注视点参数
 *  参数：
 *       [in] looper GSXR_RenderLooper 句柄, 由 GSXR_StartRenderLooper 获取
 *      [out] enabled 输出注视点渲染是否启用中, GSXR_TRUE 为启用, GSXR_FALSE 为停用
 *       [in] viewCount foveatedParams 数组个数, 需相等于 GSXR_ViewSetConfiguration 的 viewCount 数
 *      [out] foveatedParams 指向 GSXR_FoveatedParameters 结构体之数组
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Renderer_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_渲染循环类
 */
GSXR_Result GSXR_GetFoveatedParameters(
        GSXR_RenderLooper           looper,
        GSXR_Bool32*                enabled,
        uint32_t                    viewCount,
        GSXR_FoveatedParameters*    foveatedParams){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetFoveatedParameters(looper,enabled,viewCount,foveatedParams);
}