/**
 *  Created     :
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */
#include "SimpleRuntime.h"
// ------------------------------------
// GSXR_API_延伸特性类
// ------------------------------------
#include "app_api/gsxr_xrfeature.h"
/**
 *  说明：
 *      获取当前支持的所有特性类型
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *      [out] featureTypeFlags GSXR_XrFeatureType 之位掩码, 输出可支持的所有特性类型
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_延伸特性类
 */
GSXR_Result GSXR_GetSupportedXrFeatures(
        GSXR_Runtime        runtime,
        GSXR_Flags64*       featureTypeFlags){
    return GSXR_RT::SimpleRuntime::GetGRTInstance()->GRT_GetSupportedXrFeatures(runtime,featureTypeFlags);
}
/**
 *  说明：
 *      获取特性可用状态
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] featureType 指定特性类型
 *      [out] available 输出 GSXR_TRUE 表示特性可用, GSXR_FALSE 表示特性不可用
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_延伸特性类
 */
GSXR_Result GSXR_IsXrFeatureAvailable(
        GSXR_Runtime        runtime,
        GSXR_XrFeatureType  featureType,
        GSXR_Bool32*        available){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取特性之功能选项
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] featureType 指定特性类型
 *      [out] recommendedOptions 各特性之功能选项位掩码, 输出系統建議的功能选项
 *      [out] supportedOptions 各特性之功能选项位掩码, 输出可支持的所有功能选项
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      各特性之功能选项定义于该功能之头文件中
 *  所属：
 *      GSXR_API_延伸特性类
 */
GSXR_Result GSXR_GetXrFeatureOptions(
        GSXR_Runtime            runtime,
        GSXR_XrFeatureType      featureType,
        GSXR_Flags64*           recommendedOptions,
        GSXR_Flags64*           supportedOptions){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      起始特性运行
 *  参数：
 *       [in] runtime GSXR_Runtime 句柄, 由 GSXR_CreateRuntime 获取
 *       [in] startInfo 指向 GSXR_XrFeatureStartInfo 结构体, 内含特性起始资讯
 *      [out] feature 特性起始成功输出一有效句柄, 起始失败输出 GSXR_NULL_HANDLE
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Function_Unsupported
 *      GSXR_Error_Feature_Unsupported
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Out_Of_Memory
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      特性状态需处于 GSXR_EventType_FeatureState_Available 方能起始特性运行
 *  所属：
 *      GSXR_API_延伸特性类
 */
GSXR_Result GSXR_StartXrFeature(
        GSXR_Runtime                    runtime,
        const GSXR_XrFeatureStartInfo*  startInfo,
        GSXR_XrFeature*                 feature){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      停止特性运行
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_延伸特性类
 */
GSXR_Result GSXR_StopXrFeature(
        GSXR_XrFeature        feature){
    return GSXR_Result::GSXR_Result_Success;
}
/**
 *  说明：
 *      获取特性属性
 *  参数：
 *       [in] feature GSXR_XrFeature 句柄, 由 GSXR_StartXrFeature 获取
 *      [out] properties 指向 GSXR_XrFeatureProperties 结构体
 *  返回值：
 *      GSXR_Result_Success
 *      GSXR_Error_Operation_Failed
 *      GSXR_Error_Handle_Invalid
 *      GSXR_Error_Argument_Invalid
 *      GSXR_Error_Runtime_Unavailable
 *      GSXR_Error_Feature_Unavailable
 *  备注：
 *      无
 *  所属：
 *      GSXR_API_延伸特性类
 */
GSXR_Result GSXR_GetXrFeatureProperties(
        GSXR_XrFeature                feature,
        GSXR_XrFeatureProperties*     properties){
    return GSXR_Result::GSXR_Result_Success;
}