
/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include <app_api/gsxr_andros.h>
#include "SimpleRuntime.h"
#include "cassert"
#include "thread"
#include "utils/simple_log.h"
#include "utils/SimpleEventManager.h"
#include "jni.h"

#include "platform/SimpleAndroidPlatform.h"
#include "devices/SimpleInput.h"
#include "utils/SimpleMath.h"
#include "render/SimpleRender.h"
#include "extend/SimpleXrFeature.h"

using namespace GSXR_RT;
using namespace SIMPLE_EVENT;
using namespace SIMPLE_PLATFORM;
using namespace SIMPLE_RENDER;
using namespace SIMPLE_INPUT;
using namespace SIMPLE_MATH;
using namespace SIMPLE_EXTEND;

/**-----------------------
 * Runtime
 * ---------------------*/
SimpleRuntime::SimpleRuntime(){
    RT_LOGI("simple runtime %p",this);
}
SimpleRuntime::~SimpleRuntime() noexcept {
    Clear();
}

SimpleRuntime *SimpleRuntime::GetGRTInstance() {
    static SimpleRuntime simpleRuntime;
    return &simpleRuntime;
}

void SimpleRuntime::Clear() {
    RT_LOGI("clear.");
}

GSXR_Result SimpleRuntime::GRT_ResultToString(GSXR_Result result, char *string) {
    return GSXR_Result_Success;
}

GSXR_Result
SimpleRuntime::GRT_CreateRuntime(const GSXR_RuntimeCreateInfo *createInfo, GSXR_Runtime *runtime) {
    RT_LOGI("Create runtime.");
    assert(createInfo);
    assert(createInfo->systemBinding);
    SimpleAndroidPlatform::GetAndroid()->Init(createInfo);
    *runtime = (GSXR_Runtime)this;
    RT_LOGI("Create runtime ok.");
    return GSXR_Result_Success;
}

GSXR_Result
SimpleRuntime::GRT_DestroyRuntime(GSXR_Runtime runtime) {
    RT_LOGI("Destroy runtime.");
    assert(runtime == this);
    return GSXR_Result_Success;
}

GSXR_Result
SimpleRuntime::GRT_GetRuntimeProperties(GSXR_Runtime runtime,
                                                    GSXR_RuntimeProperties *runtimeProperties) {
    RT_LOGI("Get runtime properties.");
    assert(runtime == this);
    runtimeProperties->gsxrVersion=1;
    runtimeProperties->runtimeVersion=0;
    strcpy(runtimeProperties->runtimeName,"SimpleRuntime");
    return GSXR_Result_Success;
}

GSXR_Result
SimpleRuntime::GRT_PollEvent(GSXR_Runtime runtime, GSXR_Event *event) {
    RT_LOGI("Get events.");
    assert(runtime == this);
    if(0!=SimpleEventManager::GetEventManager()->PollAppEvent(event)){
        return GSXR_Result_Event_Unavailable;
    }
    return GSXR_Result_Success;
}

GSXR_Result
SimpleRuntime::GRT_CreateRenderer(GSXR_Runtime runtime, const GSXR_RendererCreateInfo *createInfo,
                                  GSXR_Renderer *renderer) {
    RT_LOGI("create render.");
    assert(runtime == this);
    return SimpleRender::GetRender()->CreateRenderer(createInfo,renderer);
}

GSXR_Result
SimpleRuntime::GRT_DestroyRenderer(GSXR_Renderer renderer) {
    RT_LOGI("Destroy render.");
    return SimpleRender::GetRender()->DestroyRenderer(renderer);
}

GSXR_Result
SimpleRuntime::GRT_GetViewSetConfigurations(GSXR_Runtime runtime, uint32_t count,
                                                        GSXR_ViewSetConfiguration *configurations) {
    RT_LOGI("get view configs.");
    return SimpleRender::GetRender()->GetViewSetConfigurations(count,configurations);
}

GSXR_Result
SimpleRuntime::GRT_GetSupportedTextureFormats(GSXR_Renderer renderer, uint32_t formatCount,
                                              GSXR_TextureFormat *formats) {
    return SimpleRender::GetRender()->GetSupportedTextureFormats(renderer,formatCount,formats);
}
GSXR_Result
SimpleRuntime::GRT_GetViewSetCount(GSXR_Runtime runtime, uint32_t *count) {
    RT_LOGI("Create texture queue.");
    assert(runtime == this);
    return SimpleRender::GetRender()->GetViewSetCount(count);
}
GSXR_Result
SimpleRuntime::GRT_CreateTextureQueue(GSXR_Renderer renderer,
                                                  const GSXR_TextureQueueCreateInfo *createInfo,
                                                  GSXR_TextureQueue *textureQueue) {
    RT_LOGI("Create texture queue.");

    return SimpleRender::GetRender()->CreateTextureQueue(renderer,createInfo,textureQueue);
}

GSXR_Result
SimpleRuntime::GRT_DestroyTextureQueue(GSXR_TextureQueue textureQueue) {

    return SimpleRender::GetRender()->DestroyTextureQueue(textureQueue);
}

GSXR_Result
SimpleRuntime::GRT_GetTextureCount(GSXR_TextureQueue textureQueue, uint32_t *count) {


    return SimpleRender::GetRender()->GetTextureCount(textureQueue,count);
}

GSXR_Result
SimpleRuntime::GRT_WaitAndAcquireAvailableTexture(GSXR_TextureQueue textureQueue,
                                                              GSXR_Duration timeout,
                                                              GSXR_TextureElement *textureElement) {

    return SimpleRender::GetRender()->WaitAndAcquireAvailableTexture(textureQueue,timeout,textureElement);
}

GSXR_Result
SimpleRuntime::GRT_ReleaseTexture(GSXR_TextureQueue textureQueue,
                                              const GSXR_TextureElement *textureElement) {

    return SimpleRender::GetRender()->ReleaseTexture(textureQueue,textureElement);
}

GSXR_Result
SimpleRuntime::GRT_SetRenderLooperThread(GSXR_RenderLooper looper, uint32_t threadId) {


    return SimpleRender::GetRender()->SetRenderLooperThread(looper,threadId);
}

GSXR_Result
SimpleRuntime::GRT_StartRenderLooper(GSXR_Renderer renderer,
                                                 const GSXR_RenderLooperStartInfo *startInfo,
                                                 GSXR_RenderLooper *looper) {
    RT_LOGI("Start render loop");
    SimpleInput::GetInput()->Start();
    return SimpleRender::GetRender()->StartRenderLooper(renderer,startInfo,looper);
}

GSXR_Result
SimpleRuntime::GRT_SubmitFrame(GSXR_RenderLooper looper, const GSXR_FrameSubmitInfo *submitInfo) {
    RT_LOGI("Submit frame.");
    SimpleEventManager::GetEventManager()->PollAllEvents();
    return SimpleRender::GetRender()->SubmitFrame(looper,submitInfo);
}

GSXR_Result
SimpleRuntime::GRT_GetTextureQueueOptions(GSXR_Renderer renderer, GSXR_Flags64 *recommendedOptions,
                                          GSXR_Flags64 *supportedOptions) {
    RT_LOGI("Get texture queue options.");

    return SimpleRender::GetRender()->GetTextureQueueOptions(renderer,recommendedOptions,supportedOptions);
}

GSXR_Result
SimpleRuntime::GRT_GetRenderFunctions(GSXR_RenderLooper looper, GSXR_Flags64 *recommendedFunctions,
                                      GSXR_Flags64 *supportedFunctions) {
    RT_LOGI("Get render functions.");

    return SimpleRender::GetRender()->GetRenderFunctions(looper,recommendedFunctions,supportedFunctions);
}

GSXR_Result
SimpleRuntime::GRT_GetSupportedXrFeatures(GSXR_Runtime runtime, GSXR_Flags64 *featureTypeFlags) {
    RT_LOGI("Get support xr features.");
    assert(runtime == this);

    return SimpleXrFeature::GetXrFeature()->GetSupportedXrFeatures(featureTypeFlags);
}

GSXR_Result
SimpleRuntime::GRT_IsXrFeatureAvailable(
        GSXR_Runtime        runtime,
        GSXR_XrFeatureType  featureType,
        GSXR_Bool32*        available){
    assert(runtime == this);

    return SimpleXrFeature::GetXrFeature()->IsXrFeatureAvailable(featureType,available);
}
GSXR_Result
SimpleRuntime::GRT_GetXrFeatureOptions(
        GSXR_Runtime            runtime,
        GSXR_XrFeatureType      featureType,
        GSXR_Flags64*           recommendedOptions,
        GSXR_Flags64*           supportedOptions){
    assert(runtime == this);

    return SimpleXrFeature::GetXrFeature()->GetXrFeatureOptions(featureType,recommendedOptions,supportedOptions);
}
GSXR_Result
SimpleRuntime::GRT_StartXrFeature(
        GSXR_Runtime                    runtime,
        const GSXR_XrFeatureStartInfo*  startInfo,
        GSXR_XrFeature*                 feature){
    assert(runtime == this);

    return SimpleXrFeature::GetXrFeature()->StartXrFeature(startInfo,feature);
}
GSXR_Result SimpleRuntime::GRT_StopXrFeature(
        GSXR_XrFeature        feature){

    return SimpleXrFeature::GetXrFeature()->StopXrFeature(feature);
}
GSXR_Result
SimpleRuntime::GRT_GetXrFeatureProperties(
        GSXR_XrFeature                feature,
        GSXR_XrFeatureProperties*     properties){

    return SimpleXrFeature::GetXrFeature()->GetXrFeatureProperties(feature,properties);
}

GSXR_Result
SimpleRuntime::GRT_GetViewPoseState(GSXR_RenderLooper looper, GSXR_Time specifiedTime,
                                                uint32_t viewCount,
                                                GSXR_ViewPoseState *viewPoseState) {
    RT_LOGI("Get head view pose. viewCount %d",viewCount);
    return SimpleRender::GetRender()->GetViewPoseState(looper,specifiedTime,viewCount,viewPoseState);
}

GSXR_Result
SimpleRuntime::GRT_WaitFrame(GSXR_RenderLooper looper, GSXR_Time *predictedDisplayTime) {
    RT_LOGI("Waite frame.");

    return SimpleRender::GetRender()->WaitFrame(looper,predictedDisplayTime);
}

GSXR_Result
SimpleRuntime::GRT_StopRenderLooper(GSXR_RenderLooper looper) {
    RT_LOGI("Stop render looper.");
    SimpleInput::GetInput()->Stop();
    return SimpleRender::GetRender()->StopRenderLooper(looper);
}

GSXR_Result
SimpleRuntime::GRT_PreRenderFrame(GSXR_RenderLooper looper,
                                              const GSXR_FrameRenderInfo *preRenderInfo) {

    return SimpleRender::GetRender()->PreRenderFrame(looper,preRenderInfo);
}

GSXR_Result
SimpleRuntime::GRT_PostRenderFrame(GSXR_RenderLooper looper,
                                               const GSXR_FrameRenderInfo *postRenderInfo) {

    return SimpleRender::GetRender()->PostRenderFrame(looper,postRenderInfo);
}

GSXR_Result
SimpleRuntime::GRT_GetFoveatedParameters(GSXR_RenderLooper looper, GSXR_Bool32 *enabled,
                                                     uint32_t viewCount,
                                                     GSXR_FoveatedParameters *foveatedParams) {

    return SimpleRender::GetRender()->GetFoveatedParameters(looper,enabled,viewCount,foveatedParams);
}

GSXR_Result
SimpleRuntime::GRT_SetPerformanceLevels(
        GSXR_Runtime runtime,
        GSXR_PerformanceLevel cpuPerfLevel,
        GSXR_PerformanceLevel gpuPerfLevel) {
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}

GSXR_Result
SimpleRuntime::GRT_SetSystemSetting(
        GSXR_Runtime runtime,
        GSXR_XrSystemSettingType settingType) {
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}

GSXR_Result
SimpleRuntime::GRT_AcquireControllerModelData(
        GSXR_XrFeature feature,
        GSXR_ControllerModelData **controllerModel,
        uint32_t *controllerModelSize) {
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}

void SimpleRuntime::GRT_ReleaseControllerModelData(
        GSXR_XrFeature feature,
        GSXR_ControllerModelData *controllerModel) {
}
GSXR_Result
SimpleRuntime::GRT_GetSupportedDevices(
        GSXR_Runtime        runtime,
        GSXR_Flags64*       deviceTypeFlags){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetSupportedDevices(deviceTypeFlags);
}
GSXR_Result
SimpleRuntime::GRT_GetPrimaryDeviceType(
        GSXR_Runtime        runtime,
        GSXR_Flags64*       deviceTypeFlags){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetPrimaryDeviceType(deviceTypeFlags);
}
GSXR_Result
SimpleRuntime::GRT_SetPrimaryDeviceType(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType){
    assert(runtime == this);

    return SimpleInput::GetInput()->SetPrimaryDeviceType(deviceType);
}
GSXR_Result
SimpleRuntime::GRT_GetSupportedDeviceInputType(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType,
        GSXR_Flags64*       deviceInputTypeFlag){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetSupportedDeviceInputType(deviceType,deviceInputTypeFlag);
}
GSXR_Result
SimpleRuntime::GRT_IsDeviceConnected(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType,
        GSXR_Bool32*        connected){
    assert(runtime == this);

    return SimpleInput::GetInput()->IsDeviceConnected(deviceType,connected);
}
GSXR_Result
SimpleRuntime::GRT_GetDisplay(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType,
        GSXR_DisplayId*     displayId){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetDisplay(deviceType,displayId);
}
GSXR_Result
SimpleRuntime::GRT_GetInputIdConfiguration(
        GSXR_Runtime                runtime,
        GSXR_DeviceType             deviceType,
        GSXR_Device_InputType       inputType,
        int                         inputId,
        GSXR_Flags64*               configuration){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetInputIdConfiguration(deviceType,inputType,inputId,configuration);
}
GSXR_Result
SimpleRuntime::GRT_GetInputClickStates(
        GSXR_Runtime                runtime,
        GSXR_DeviceType             deviceType,
        GSXR_Device_InputType       inputType,
        GSXR_Flags64*               clicks){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetInputClickStates(deviceType,inputType,clicks);
}
GSXR_Result
SimpleRuntime::GRT_GetInputTouchStates(
        GSXR_Runtime                runtime,
        GSXR_DeviceType             deviceType,
        GSXR_Device_InputType       inputType,
        GSXR_Flags64*               touchs){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetInputTouchStates(deviceType,inputType,touchs);
}
GSXR_Result
SimpleRuntime::GRT_GetInputAnalogState(
        GSXR_Runtime                runtime,
        GSXR_DeviceType             deviceType,
        GSXR_Device_InputType       inputType,
        int                         inputId,
        GSXR_Vector2f*              analog){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetInputAnalogState(deviceType,inputType,inputId,analog);
}
GSXR_Result
SimpleRuntime::GRT_TriggerVibration(
        GSXR_Runtime                    runtime,
        GSXR_DeviceType                 deviceType,
        GSXR_Bool32                     manual,
        const GSXR_HapticVibration*     haptics){
    assert(runtime == this);

    return SimpleInput::GetInput()->TriggerVibration(deviceType,manual,haptics);
}
GSXR_Result
SimpleRuntime::GRT_GetDegreeOfFreedom(
        GSXR_Runtime            runtime,
        GSXR_DeviceType         deviceType,
        GSXR_DegreeOfFreedom*   dof){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetDegreeOfFreedom(deviceType,dof);
}
GSXR_Result
SimpleRuntime::GRT_SetSpaceOrigin(
        GSXR_Runtime        runtime,
        GSXR_SpaceOrigin    space,
        const GSXR_Posef*   offset){
    assert(runtime == this);

    return SimpleInput::GetInput()->SetSpaceOrigin(space,offset);
}
GSXR_Result
SimpleRuntime::GRT_ResetOrientationAndPosition(
        GSXR_Runtime        runtime,
        float               Yaw){
    assert(runtime == this);

    return SimpleInput::GetInput()->ResetOrientationAndPosition(Yaw);
}
GSXR_Result
SimpleRuntime::GRT_GetPoseState(
        GSXR_Runtime        runtime,
        GSXR_DeviceType     deviceType,
        GSXR_Time           specifiedTime,
        GSXR_PoseState*     poseState){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetPoseState(deviceType,specifiedTime,poseState);
}
GSXR_Result
SimpleRuntime::GRT_GetSpaceBoundsConfiguration(
        GSXR_Runtime                    runtime,
        GSXR_SpaceBoundsConfiguration*  boundsConfiguration){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetSpaceBoundsConfiguration(boundsConfiguration);
}
GSXR_Result
SimpleRuntime::GRT_GetSpaceBoundsRectangle(
        GSXR_Runtime                    runtime,
        float*                          width,
        float*                          length){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetSpaceBoundsRectangle(width,length);
}
GSXR_Result
SimpleRuntime::GRT_GetSpaceBoundsCircle(
        GSXR_Runtime                    runtime,
        float*                          diameter){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetSpaceBoundsCircle(diameter);
}
GSXR_Result
SimpleRuntime::GRT_GetSpaceBoundsIrregular(
        GSXR_Runtime                    runtime,
        uint32_t                        pointCount,
        GSXR_Posef*                     pointArray){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetSpaceBoundsIrregular(pointCount,pointArray);
}
GSXR_Result
SimpleRuntime::GRT_SetSpaceBoundsVisible(
        GSXR_Runtime                    runtime,
        GSXR_SpaceBoundsVisible         visible){
    assert(runtime == this);

    return SimpleInput::GetInput()->SetSpaceBoundsVisible(visible);
}
GSXR_Result
SimpleRuntime::GRT_GetDeviceProperties(
        GSXR_Runtime            runtime,
        GSXR_DeviceType         deviceType,
        GSXR_DeviceProperties*  deviceProperties){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetDeviceProperties(deviceType,deviceProperties);
}
GSXR_Result
SimpleRuntime::GRT_GetBatteryStatus(
        GSXR_Runtime            runtime,
        GSXR_DeviceType         deviceType,
        GSXR_BatteryStatus*     batteryStatus){
    assert(runtime == this);

    return SimpleInput::GetInput()->GetBatteryStatus(deviceType,batteryStatus);
}

GSXR_Result
SimpleRuntime::GRT_GetSupportedEyeFactors(
        GSXR_XrFeature      feature,
        GSXR_Flags64*       factors){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_SetEyeFactorRequest(
        GSXR_XrFeature      feature,
        GSXR_Flags64        request){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}

GSXR_Result
SimpleRuntime::GRT_GetEyeTrackingData(
        GSXR_XrFeature              feature,
        GSXR_Time                   specifiedTime,
        GSXR_EyeTrackingData*       data){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}

GSXR_Result
SimpleRuntime::GRT_AcquireFaceTrackerData(
        GSXR_XrFeature                          feature,
        const GSXR_FaceTrackerAcquireInfo*      acquireInfo,
        GSXR_FaceTrackerData**                  faceTracker){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_ReleaseFaceTrackerData(
        GSXR_XrFeature              feature,
        GSXR_FaceTrackerData*       faceTracker){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_AcquireFaceMatchTrackerData(
        GSXR_XrFeature                              feature,
        const GSXR_FaceMatchTrackerAcquireInfo*     acquireInfo,
        GSXR_FaceMatchTrackerData**                 faceMatchTracker
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_ReleaseFaceMatchTrackerData(
        GSXR_XrFeature                  feature,
        GSXR_FaceMatchTrackerData*      faceMatchTracker
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_GetFacialMainFeatureData(
        GSXR_XrFeature                  feature,
        GSXR_FaceId                     facelId,
        GSXR_FacialMainFeatureData*     data
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_GetFacialDetailFeatureData(
        GSXR_XrFeature                  feature,
        GSXR_FaceId                     facelId,
        GSXR_FacialDetailFeatureData*   data
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_GetSupportedHandGestures(
        GSXR_XrFeature      feature,
        GSXR_Flags64*       gestures
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_SetHandGestureRequest(
        GSXR_XrFeature      feature,
        GSXR_Flags64        request
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_GetHandGestureData(
        GSXR_XrFeature              feature,
        GSXR_HandGestureData*       data
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_RegisterHandGestureEventCallback(
        GSXR_XrFeature                  feature,
        GSXR_HandGestureEventCallback   callback,
        void*                           cookie
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_GetTrackedHandJoints(
        GSXR_XrFeature      feature,
        GSXR_Flags64*       joints
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_GetHandJointValidity(
        GSXR_XrFeature      feature,
        GSXR_Flags64*       validity
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_GetHandTrackingData(
        GSXR_XrFeature              feature,
        GSXR_Time                   specifiedTime,
        GSXR_HandTrackingData*      data
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}

GSXR_Result
SimpleRuntime::GRT_LoadModelImage(
        GSXR_XrFeature          feature,
        const GSXR_ImageInfo*   imageInfo,
        GSXR_ImageId*           modelId
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_UnloadModelImage(
        GSXR_XrFeature      feature,
        GSXR_ImageId        modelId
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_AcquireImageTrackerData(
        GSXR_XrFeature                          feature,
        const GSXR_ImageTrackerAcquireInfo*     acquireInfo,
        GSXR_ImageTrackerData**                 imageTracker
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_ReleaseImageTrackerData(
        GSXR_XrFeature              feature,
        GSXR_ImageTrackerData*      imageTracker
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}


GSXR_Result
SimpleRuntime::GRT_AcquirePlaneTrackerData(
        GSXR_XrFeature              feature,
        GSXR_PlaneTrackerData**     planeTracker
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_ReleasePlaneTrackerData(
        GSXR_XrFeature              feature,
        GSXR_PlaneTrackerData*      planeTracker
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}

GSXR_Result
SimpleRuntime::GRT_AcquirePointCloudData(
        GSXR_XrFeature          feature,
        GSXR_PointCloudData**   pointCloud
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
GSXR_Result
SimpleRuntime::GRT_ReleasePointCloudData(
        GSXR_XrFeature          feature,
        GSXR_PointCloudData*    pointCloud
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}

GSXR_Result
SimpleRuntime::GRT_InputVoiceCommand(
        GSXR_XrFeature          feature,
        GSXR_Duration           timeout
){
    return GSXR_Result::GSXR_Error_Feature_Unsupported;
}
