
/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#ifndef GSXRRUNTIME_SIMPLERUNTIME_H
#define GSXRRUNTIME_SIMPLERUNTIME_H

#include <cstdint>

#include <app_api/gsxr_system.h>
#include <app_api/gsxr_controller_model.h>
#include <app_api/gsxr_eye_tracking.h>
#include <app_api/gsxr_face_tracker.h>
#include <app_api/gsxr_hand_tracking.h>
#include <app_api/gsxr_image_tracker.h>
#include <app_api/gsxr_plane_tracker.h>
#include <app_api/gsxr_point_cloud.h>
#include "app_api/gsxr.h"

namespace GSXR_RT{
/**
 * 定义类：
 * SimpleRuntime
 * 
 */
    class SimpleRuntime {
    public:
        /**--------------------------
        * GSXR_API_通用基础类
        ------------------------* */
        GSXR_Result GRT_ResultToString(GSXR_Result result,char string[GSXR_COMMON_STRING_MAX_SIZE]);
        /**
         * 初始化XR相关事务
         * */
        GSXR_Result GRT_CreateRuntime(const GSXR_RuntimeCreateInfo* createInfo,GSXR_Runtime* runtime);
        GSXR_Result GRT_DestroyRuntime(GSXR_Runtime runtime);
        GSXR_Result GRT_GetRuntimeProperties(GSXR_Runtime runtime,GSXR_RuntimeProperties* runtimeProperties);
        GSXR_Result GRT_PollEvent(GSXR_Runtime runtime,GSXR_Event* event);
        /**--------------------------
        * GSXR_API_渲染视图类
        ------------------------* */
        GSXR_Result GRT_CreateRenderer(GSXR_Runtime runtime,const GSXR_RendererCreateInfo* createInfo,GSXR_Renderer* renderer);
        GSXR_Result GRT_DestroyRenderer(GSXR_Renderer renderer);
        GSXR_Result GRT_GetViewSetCount(GSXR_Runtime runtime,uint32_t* count);
        GSXR_Result GRT_GetViewSetConfigurations(GSXR_Runtime runtime,uint32_t count,GSXR_ViewSetConfiguration* configurations);
        GSXR_Result GRT_GetTextureQueueOptions(GSXR_Renderer renderer,GSXR_Flags64* recommendedOptions,GSXR_Flags64* supportedOptions);
        GSXR_Result GRT_GetSupportedTextureFormats(GSXR_Renderer renderer,uint32_t formatCount,GSXR_TextureFormat* formats);
        GSXR_Result GRT_CreateTextureQueue(GSXR_Renderer renderer,const GSXR_TextureQueueCreateInfo *createInfo,GSXR_TextureQueue *textureQueue);
        GSXR_Result GRT_DestroyTextureQueue(GSXR_TextureQueue textureQueue);
        GSXR_Result GRT_GetTextureCount(GSXR_TextureQueue textureQueue,uint32_t *count);
        GSXR_Result GRT_WaitAndAcquireAvailableTexture(GSXR_TextureQueue textureQueue,GSXR_Duration timeout,GSXR_TextureElement *textureElement);
        GSXR_Result GRT_ReleaseTexture(GSXR_TextureQueue textureQueue,const GSXR_TextureElement *textureElement);
        GSXR_Result GRT_StartRenderLooper(GSXR_Renderer renderer,const GSXR_RenderLooperStartInfo *startInfo,GSXR_RenderLooper *looper);
        GSXR_Result GRT_StopRenderLooper(GSXR_RenderLooper looper);
        GSXR_Result GRT_SetRenderLooperThread(GSXR_RenderLooper looper,uint32_t threadId);
        GSXR_Result GRT_WaitFrame(GSXR_RenderLooper looper,GSXR_Time *predictedDisplayTime);
        GSXR_Result GRT_GetViewPoseState(GSXR_RenderLooper looper,GSXR_Time specifiedTime,uint32_t viewCount,GSXR_ViewPoseState *viewPoseState);
        GSXR_Result GRT_SubmitFrame(GSXR_RenderLooper looper,const GSXR_FrameSubmitInfo *submitInfo);
        GSXR_Result GRT_GetRenderFunctions(GSXR_RenderLooper looper,GSXR_Flags64 *recommendedFunctions,GSXR_Flags64 *supportedFunctions);
        GSXR_Result GRT_PreRenderFrame(GSXR_RenderLooper looper,const GSXR_FrameRenderInfo *preRenderInfo);
        GSXR_Result GRT_PostRenderFrame(GSXR_RenderLooper looper,const GSXR_FrameRenderInfo *postRenderInfo);
        GSXR_Result GRT_GetFoveatedParameters(GSXR_RenderLooper looper,GSXR_Bool32 *enabled,uint32_t viewCount,GSXR_FoveatedParameters *foveatedParams);
        /**--------------------------
         * GSXR_API_系統性能类
         ------------------------* */
        GSXR_Result GRT_SetPerformanceLevels(GSXR_Runtime runtime,GSXR_PerformanceLevel cpuPerfLevel,GSXR_PerformanceLevel gpuPerfLevel);
        GSXR_Result GRT_SetSystemSetting(GSXR_Runtime runtime,GSXR_XrSystemSettingType settingType);

        /**--------------------------
        * GSXR_API_手柄模型类
        ------------------------* */
        GSXR_Result GRT_AcquireControllerModelData(GSXR_XrFeature feature,GSXR_ControllerModelData **controllerModel,uint32_t *controllerModelSize);
        void GRT_ReleaseControllerModelData(GSXR_XrFeature feature,GSXR_ControllerModelData *controllerModel);
        /**--------------------------
        * GSXR_API_设备输入类
        ------------------------* */
        GSXR_Result GRT_GetSupportedDevices(GSXR_Runtime runtime,GSXR_Flags64 *deviceTypeFlags);
        GSXR_Result GRT_GetPrimaryDeviceType(GSXR_Runtime runtime,GSXR_Flags64 *deviceTypeFlags);
        GSXR_Result GRT_SetPrimaryDeviceType(GSXR_Runtime runtime,GSXR_DeviceType deviceType);
        GSXR_Result GRT_GetSupportedDeviceInputType(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_Flags64 *deviceInputTypeFlag);
        GSXR_Result GRT_IsDeviceConnected(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_Bool32 *connected);
        GSXR_Result GRT_GetDisplay(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_DisplayId *displayId);
        GSXR_Result GRT_GetInputIdConfiguration(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,int inputId,GSXR_Flags64 *configuration);
        GSXR_Result GRT_GetInputClickStates(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,GSXR_Flags64 *clicks);
        GSXR_Result GRT_GetInputTouchStates(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,GSXR_Flags64 *touchs);
        GSXR_Result GRT_GetInputAnalogState(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_Device_InputType inputType,int inputId,GSXR_Vector2f *analog);
        GSXR_Result GRT_TriggerVibration(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_Bool32 manual,const GSXR_HapticVibration *haptics);
        GSXR_Result GRT_GetDegreeOfFreedom(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_DegreeOfFreedom *dof);
        GSXR_Result GRT_SetSpaceOrigin(GSXR_Runtime runtime,GSXR_SpaceOrigin space,const GSXR_Posef *offset);
        GSXR_Result GRT_ResetOrientationAndPosition(GSXR_Runtime runtime,float Yaw);
        GSXR_Result GRT_GetPoseState(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_Time specifiedTime,GSXR_PoseState *poseState);
        GSXR_Result GRT_GetSpaceBoundsConfiguration(GSXR_Runtime runtime,GSXR_SpaceBoundsConfiguration *boundsConfiguration);
        GSXR_Result GRT_GetSpaceBoundsRectangle(GSXR_Runtime runtime,float *width,float *length);
        GSXR_Result GRT_GetSpaceBoundsCircle(GSXR_Runtime runtime,float *diameter);
        GSXR_Result GRT_GetSpaceBoundsIrregular(GSXR_Runtime runtime,uint32_t pointCount,GSXR_Posef *pointArray);
        GSXR_Result GRT_SetSpaceBoundsVisible(GSXR_Runtime runtime,GSXR_SpaceBoundsVisible visible);
        GSXR_Result GRT_GetDeviceProperties(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_DeviceProperties *deviceProperties);
        GSXR_Result GRT_GetBatteryStatus(GSXR_Runtime runtime,GSXR_DeviceType deviceType,GSXR_BatteryStatus *batteryStatus);
        /**--------------------------
        * GSXR_API_扩展特性类
        ------------------------* */
        GSXR_Result GRT_GetSupportedXrFeatures(GSXR_Runtime runtime,GSXR_Flags64 *featureTypeFlags);
        GSXR_Result GRT_IsXrFeatureAvailable(GSXR_Runtime runtime,GSXR_XrFeatureType featureType,GSXR_Bool32 *available);
        GSXR_Result GRT_GetXrFeatureOptions(GSXR_Runtime runtime,GSXR_XrFeatureType featureType,GSXR_Flags64 *recommendedOptions,GSXR_Flags64 *supportedOptions);
        GSXR_Result GRT_StartXrFeature(GSXR_Runtime runtime,const GSXR_XrFeatureStartInfo *startInfo,GSXR_XrFeature *feature);
        GSXR_Result GRT_StopXrFeature(GSXR_XrFeature feature);
        GSXR_Result GRT_GetXrFeatureProperties(GSXR_XrFeature feature,GSXR_XrFeatureProperties *properties);

        /**--------------------------
        * GSXR_API 眼球追踪
        ------------------------* */
        GSXR_Result GRT_GetSupportedEyeFactors(GSXR_XrFeature feature,GSXR_Flags64 *factors);
        GSXR_Result GRT_SetEyeFactorRequest(GSXR_XrFeature feature,GSXR_Flags64 request);
        GSXR_Result GRT_GetEyeTrackingData(GSXR_XrFeature feature,GSXR_Time specifiedTime,GSXR_EyeTrackingData *data);
        /**--------------------------
        * GSXR_API_脸部识别类
        ------------------------* */
        GSXR_Result GRT_AcquireFaceTrackerData(GSXR_XrFeature feature,const GSXR_FaceTrackerAcquireInfo *acquireInfo,GSXR_FaceTrackerData **faceTracker);
        GSXR_Result GRT_ReleaseFaceTrackerData(GSXR_XrFeature feature,GSXR_FaceTrackerData *faceTracker);
        GSXR_Result GRT_AcquireFaceMatchTrackerData(GSXR_XrFeature feature,const GSXR_FaceMatchTrackerAcquireInfo *acquireInfo,GSXR_FaceMatchTrackerData **faceMatchTracker);
        GSXR_Result GRT_ReleaseFaceMatchTrackerData(GSXR_XrFeature feature,GSXR_FaceMatchTrackerData *faceMatchTracker);
        GSXR_Result GRT_GetFacialMainFeatureData(GSXR_XrFeature feature,GSXR_FaceId facelId,GSXR_FacialMainFeatureData *data);
        GSXR_Result GRT_GetFacialDetailFeatureData(GSXR_XrFeature feature,GSXR_FaceId facelId,GSXR_FacialDetailFeatureData *data);
        /**--------------------------
        * GSXR_API_手势识别类
        ------------------------* */
        GSXR_Result GRT_GetSupportedHandGestures(GSXR_XrFeature feature,GSXR_Flags64 *gestures);
        GSXR_Result GRT_SetHandGestureRequest(GSXR_XrFeature feature,GSXR_Flags64 request);
        GSXR_Result GRT_GetHandGestureData(GSXR_XrFeature feature,GSXR_HandGestureData *data);
        GSXR_Result GRT_RegisterHandGestureEventCallback(GSXR_XrFeature feature,GSXR_HandGestureEventCallback callback,void *cookie);
        GSXR_Result GRT_GetTrackedHandJoints(GSXR_XrFeature feature,GSXR_Flags64 *joints);
        GSXR_Result GRT_GetHandJointValidity(GSXR_XrFeature feature,GSXR_Flags64 *validity);
        GSXR_Result GRT_GetHandTrackingData(GSXR_XrFeature feature,GSXR_Time specifiedTime,GSXR_HandTrackingData *data);
        /**--------------------------
        * GSXR_API_图像识别类
        ------------------------* */
        GSXR_Result GRT_LoadModelImage(GSXR_XrFeature feature,const GSXR_ImageInfo *imageInfo,GSXR_ImageId *modelId);
        GSXR_Result GRT_UnloadModelImage(GSXR_XrFeature feature,GSXR_ImageId modelId);
        GSXR_Result GRT_AcquireImageTrackerData(GSXR_XrFeature feature,const GSXR_ImageTrackerAcquireInfo *acquireInfo,GSXR_ImageTrackerData **imageTracker);
        GSXR_Result GRT_ReleaseImageTrackerData(GSXR_XrFeature feature,GSXR_ImageTrackerData *imageTracker);
        /**--------------------------
        * GSXR_API_平面追踪类
        ------------------------* */
        GSXR_Result GRT_AcquirePlaneTrackerData(GSXR_XrFeature feature,GSXR_PlaneTrackerData **planeTracker);
        GSXR_Result GRT_ReleasePlaneTrackerData(GSXR_XrFeature feature,GSXR_PlaneTrackerData *planeTracker);
        /**--------------------------
        * GSXR_API_点云数据类
        ------------------------* */
        GSXR_Result GRT_AcquirePointCloudData(GSXR_XrFeature feature,GSXR_PointCloudData **pointCloud);
        GSXR_Result GRT_ReleasePointCloudData(GSXR_XrFeature feature,GSXR_PointCloudData *pointCloud);
        /**--------------------------
        * GSXR_API_语音命令类
        ------------------------* */
        GSXR_Result GRT_InputVoiceCommand(GSXR_XrFeature feature,GSXR_Duration timeout);

public:
        static SimpleRuntime* GetGRTInstance();
        virtual ~SimpleRuntime() noexcept;
        void Clear();
    private:
        SimpleRuntime();
    private:

    };
}//namespace GSXR_RT

#endif //GSXRRUNTIME_SIMPLERUNTIME_H
