/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#include <app_api/gsxr_andros.h>
#include "SimpleAndroidPlatform.h"
#include "utils/simple_log.h"
#include "devices/SimpleInput.h"

#include <cassert>
#include <sys/prctl.h>

#define DEX_FILE  "/sdcard/gsxr/gsxrdex/GsxrApi.dex"
#define GSXR_API_CLASS  "com/gsxr/simpleruntime/GsxrJavaApi"

using namespace SIMPLE_PLATFORM;
using namespace SIMPLE_INPUT;
//---------定义屏幕触摸时间JNI函数
void TouchUpCallBack(JNIEnv* env, jobject){
    SimpleInput::GetInput()->TouchUpEvent();
}
void TouchDownCallBack(JNIEnv* env, jobject){
    SimpleInput::GetInput()->TouchDownEvent();
}
void TouchLeftCallBack(JNIEnv* env, jobject){
    SimpleInput::GetInput()->TouchLeftEvent();
}
void TouchRightCallBack(JNIEnv* env, jobject){
    SimpleInput::GetInput()->TouchRightEvent();
}
void SingleTapCallBack(JNIEnv* env, jobject){
    SimpleInput::GetInput()->SingleTapEvent();
}
void UpdateTouchDataCallBack(JNIEnv* env, jobject, jfloatArray inData){
    jfloat jdata[4] = {};
    env->GetFloatArrayRegion(inData,0,4,jdata);
    float outData[4]= {jdata[0],jdata[1],jdata[2],jdata[3]};
    SimpleInput::GetInput()->UpdateTouchData(outData);
}
void SingleTouchDownEventCallBack(JNIEnv* env, jobject, jboolean inData){
    SimpleInput::GetInput()->SingleTouchDownEvent(inData);
}
//-----------------

SimpleAndroidPlatform *SimpleAndroidPlatform::GetAndroid() {
    static SimpleAndroidPlatform androidPlatform;
    return &androidPlatform;
}
SimpleAndroidPlatform::SimpleAndroidPlatform():gsxrJniUtils(nullptr),javaReady(false) {
    gsxrJniUtils = new GsxrJniUtils();
}
SimpleAndroidPlatform::~SimpleAndroidPlatform() {
    if(gsxrJniUtils){
        delete gsxrJniUtils;
    }
}

int32_t SimpleAndroidPlatform::Init(const GSXR_RuntimeCreateInfo* createInfo) {
    RT_LOGD("Init");
    auto gsxrAndrosSystemBinding = (GSXR_AndrosSystemBinding*)createInfo->systemBinding;
    gsxrJniUtils->javaVm = (JavaVM*)gsxrAndrosSystemBinding->androsAppVM;
    gsxrJniUtils->activity = (jobject)gsxrAndrosSystemBinding->androsAppActivity;
    RT_LOGD("activity %p",gsxrJniUtils->activity);
    javaReady = (InitJava()==0) ? true : false;
    assert(gsxrJniUtils->gsxrJavaApi);
    return 0;
}

void SimpleAndroidPlatform::Destroy() {

}

int32_t SimpleAndroidPlatform::InitJava() {
    RT_LOGD("InitJava");
    assert(gsxrJniUtils->javaVm);
    char name[128]={0};
    prctl(PR_GET_NAME, (char *)(name));
    JavaVMAttachArgs args;
    args.version = JNI_VERSION_1_4;
    args.name = (char*)name;
    args.group = nullptr;
    if (gsxrJniUtils->javaVm->AttachCurrentThread(&(gsxrJniUtils->jniEnv), &args) != JNI_OK)
    {
        RT_LOGE("javaVm AttachCurrentThread failed.");
    }
    assert(gsxrJniUtils->jniEnv);
    JNIEnv* env = gsxrJniUtils->jniEnv;
    jclass classloaderClass =env->FindClass("java/lang/ClassLoader");
    jmethodID getSystemClassLoader =env->GetStaticMethodID(classloaderClass,
                                                           "getSystemClassLoader",
                                                           "()Ljava/lang/ClassLoader;");
    jobject loader =env->CallStaticObjectMethod(classloaderClass,getSystemClassLoader);
    jclass dexLoaderClass =env->FindClass("dalvik/system/DexClassLoader");
    jmethodID initDexLoaderMethod =env->GetMethodID(
            dexLoaderClass, "<init>",
            "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/ClassLoader;)V");
    if(NULL == initDexLoaderMethod){
        RT_LOGE("get initDexLoaderMethod NULL");
        return -1;
    }
    jstring dexPath = env->NewStringUTF(DEX_FILE);
    auto dexLoader =env->NewObject(dexLoaderClass,initDexLoaderMethod, dexPath,
                                   NULL, NULL, loader);
    if(NULL == dexLoader){
        RT_LOGE("new dexLoader NULL");
        return -1;
    }
    auto findclassMethod =env->GetMethodID(dexLoaderClass,"findClass",//findClass
                                           "(Ljava/lang/String;)Ljava/lang/Class;");
    if(NULL==findclassMethod){
        RT_LOGD("no findclass, get loadclass");
        findclassMethod =env->GetMethodID(dexLoaderClass,"loadClass",
                                          "(Ljava/lang/String;)Ljava/lang/Class;");
    }
    if(NULL == findclassMethod){
        RT_LOGE("get findclassMethod NULL");
        return -1;
    }
    jstring strClassName = env->NewStringUTF(GSXR_API_CLASS);
    gsxrJniUtils->gsxrJavaApiClass =(jclass)env->CallObjectMethod(dexLoader,findclassMethod,strClassName);
    if(NULL == gsxrJniUtils->gsxrJavaApiClass){
        RT_LOGE("get GsxrJavaApi Class is NULL");
        return -1;
    }
    jmethodID GetGsxrApiInstanceMethodId = gsxrJniUtils->jniEnv->GetStaticMethodID(
            gsxrJniUtils->gsxrJavaApiClass,
            "GetGsxrApiInstance",
            "()Lcom/gsxr/simpleruntime/GsxrJavaApi;");
    if(NULL == GetGsxrApiInstanceMethodId){
        RT_LOGE("GetGsxrApiInstance function is NULL.");
        return -1;
    }
    RT_LOGD("CallStaticObjectMethod GetGsxrApiInstance");
    gsxrJniUtils->gsxrJavaApi = gsxrJniUtils->jniEnv->CallStaticObjectMethod(
            gsxrJniUtils->gsxrJavaApiClass,
            GetGsxrApiInstanceMethodId);
    assert(gsxrJniUtils->gsxrJavaApi);
    jmethodID GetRegisterViewTouchMethodId = gsxrJniUtils->jniEnv->GetMethodID(
            gsxrJniUtils->gsxrJavaApiClass,
            "RegisterViewTouch",
            "(Landroid/app/Activity;)V");
    if(NULL == GetRegisterViewTouchMethodId){
        RT_LOGE("RegisterViewTouch function is NULL.");
        return -1;
    }
    RT_LOGD("CallVoidMethod RegisterViewTouch");
    gsxrJniUtils->jniEnv->CallVoidMethod(gsxrJniUtils->gsxrJavaApi,
                                         GetRegisterViewTouchMethodId,
                                         gsxrJniUtils->activity);
    const JNINativeMethod methods[] = {
            {
                    (char*)"touchUpCallBack",
                    (char*)"()V", reinterpret_cast<void*>(TouchUpCallBack)
            },
            {
                    (char*)"touchDownCallBack",
                    (char*)"()V", reinterpret_cast<void*>(TouchDownCallBack)
            },
            {
                    (char*)"touchLeftCallBack",
                    (char*)"()V", reinterpret_cast<void*>(TouchLeftCallBack)
            },
            {
                    (char*)"touchRightCallBack",
                    (char*)"()V", reinterpret_cast<void*>(TouchRightCallBack)
            },
            {
                    (char*)"singleTapCallBack",
                    (char*)"()V", reinterpret_cast<void*>(SingleTapCallBack)
            },
            {
                    (char*)"updateTouchDataCallBack",
                    (char*)"([F)V", reinterpret_cast<void*>(UpdateTouchDataCallBack)
            },
            {
                    (char*)"SingleTouchDownEventCallBack",
                    (char*)"(Z)V", reinterpret_cast<void*>(SingleTouchDownEventCallBack)
            },
    };
    RT_LOGD("RegisterNatives");
    int32_t ret = gsxrJniUtils->jniEnv->RegisterNatives(gsxrJniUtils->gsxrJavaApiClass,methods,
                                           sizeof (methods)/sizeof (JNINativeMethod));
    RT_LOGD("InitJava ok.");
    return 0;
}