

#ifndef GSXRRUNTIME_SIMPLEANDROIDPLATFORM_H
#define GSXRRUNTIME_SIMPLEANDROIDPLATFORM_H
/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */
#include <app_api/gsxr.h>
#include "jni.h"
namespace SIMPLE_PLATFORM{
    /**
    *
     * 定义一些负责收集和调用与系统平台相关的数据结构和函数来服务GSXR的实现运转。本实现中通过GSXR应用调用了GSXR API
     * 接口函数GSXR_CreateRuntime**传递了**GSXR_AndrosSystemBinding**结构体数据，获取到了GSXR应用的Java虚拟机
     * 指针以及当前的Activity对象。通过这些信息Runtime可以通过JNI机制与系统框架进行通信，获取Runtime想要获
     * 取的系统信息，例如屏幕尺寸大小，屏幕刷新频率，系统版本等等。
     *
     */

    /**
     * 屏幕的尺寸数据结构体
     */
    typedef struct DisplayInfo{
        uint32_t width;
        uint32_t height;
    }DisplayInfo;

    /**
     * 定义Jni相关的各种功能函数及类别
     * */
     typedef struct GsxrJniUtils{
         JavaVM* javaVm;
         JNIEnv* jniEnv;
         jobject activity;
         DisplayInfo displayInfo;
         jclass gsxrJavaApiClass;
         jobject gsxrJavaApi;
     }GsxrJniUtils;

    class SimpleAndroidPlatform{
    public:
        static SimpleAndroidPlatform* GetAndroid();
        virtual ~SimpleAndroidPlatform();
        int32_t Init(const GSXR_RuntimeCreateInfo*);
        void Destroy();
    private:
        SimpleAndroidPlatform();
        int32_t InitJava();
    private:
        GsxrJniUtils* gsxrJniUtils;
        bool javaReady;
    };
}
#endif //GSXRRUNTIME_SIMPLEANDROIDPLATFORM_H
