/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */
#include <cstring>
#include <algorithm>
#include "SimpleEventManager.h"
#include "simple_log.h"

using namespace SIMPLE_EVENT;

SimpleEventManager *SimpleEventManager::GetEventManager() {
    static SimpleEventManager eventManager;
    return &eventManager;
}
SimpleEventManager::~SimpleEventManager() {
    RT_LOGI("Release event Manager.");
    for(int32_t i=0; i<EVENT_QUEUES;i++)
    {
        if(rtEventQueue[i]){
            delete rtEventQueue[i];
            rtEventQueue[i] = nullptr;
        }
    }
    delete appEventQueue;
    appEventQueue = nullptr;
}
SimpleEventManager::SimpleEventManager() {
    RT_LOGI("Create event Manager.");
    for(int32_t i=0; i<EVENT_QUEUES;i++)
    {
        rtEventQueue[i] = nullptr;
    }
    clock_gettime(CLOCK_REALTIME, &timeBase);
    appEventQueue = new SimpleEventQueue(EVENT_QUEUES * EVENTS_PER_QUEUE, timeBase);
}

int32_t SimpleEventManager::SubmitEvent(GSXR_Event * event) {
    pid_t threadId = gettid();
    SimpleEventQueue* eventQueue= nullptr;
    for (int32_t i = 0; i < EVENT_QUEUES; i++)
    {
        if (rtEventQueue[i] != nullptr && rtEventQueue[i]->threadId == threadId)
        {
            eventQueue = rtEventQueue[i];
        }
    }
    if(nullptr==eventQueue){
        for (int32_t i = 0; i < EVENT_QUEUES; i++)
        {
            if (rtEventQueue[i] == nullptr)
            {
                rtEventQueue[i] = new SimpleEventQueue (EVENTS_PER_QUEUE, timeBase);
                rtEventQueue[i]->threadId = threadId;
                eventQueue = rtEventQueue[i];
                break;
            }
        }
    }
    if(nullptr==eventQueue){
        RT_LOGE("Event queue is full!");
        return -1;
    }
    if(0 != eventQueue->eventQueueBuffer->EnqueueEventBuffer(event)){
        RT_LOGE("Event enqueue failed!");
        return -1;
    }
    return 0;
}

int32_t SimpleEventManager::PollAppEvent(GSXR_Event *event) {
    if(appEventQueue){
        if(0== appEventQueue->eventQueueBuffer->DequeueEventBuffer(event)){
            return 0;
        }else{
            RT_LOGD("App event dequeue is unavailable!");
            return -1;
        }
    }else{
        RT_LOGE("App event queue is null!");
        return -1;
    }
    return 0;
}
const int32_t num = EVENTS_PER_QUEUE * EVENT_QUEUES;
void SimpleEventManager::PollAllEvents() {
    RT_LOGD("Poll all events.");
    static GSXR_Event events[num];
    memset(events, 0, sizeof(GSXR_Event) * num);
    int32_t eventIndex=0;
    for (int32_t i = 0; i < EVENT_QUEUES; i++)
    {
        if (rtEventQueue[i] != nullptr)
        {
            while (0 == (rtEventQueue[i]->eventQueueBuffer->DequeueEventBuffer(&events[eventIndex])))
            {
                eventIndex++;
            }
        }
    }
    if (eventIndex > 0)
    {
//        std::sort(events, events + eventIndex, [](GSXR_Event& e0,GSXR_Event& e1)->bool {
//            return (e0.timestamp < e0.timestamp);
//        });
        for (int32_t i = 0; i < eventIndex; i++)
        {
            appEventQueue->eventQueueBuffer->EnqueueEventBuffer(&events[i]);
        }
    }
}