/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */

#ifndef GSXRRUNTIME_SIMPLEEVENTMANAGER_H
#define GSXRRUNTIME_SIMPLEEVENTMANAGER_H
#include <type_traits>
#include <atomic>
#include <cassert>
#include <ctime>
#include <unistd.h>
#include "app_api/gsxr.h"
#include "utils/simple_log.h"
#define EVENT_QUEUES 16
#define EVENTS_PER_QUEUE 32
namespace SIMPLE_EVENT{
    typedef typename
    std::aligned_storage<sizeof(GSXR_Event),std::alignment_of<GSXR_Event>::value>::type AlignBuffer;
    typedef struct SimpleEventQueueBuffer{
        GSXR_Event* const eventBuffer;
        const size_t eventSize;
        const size_t eventMask;
        std::atomic<size_t> bufferHead;
        std::atomic<size_t> bufferTail;
        SimpleEventQueueBuffer(size_t size):
                eventBuffer(reinterpret_cast<GSXR_Event*>(new AlignBuffer[size+1])),
                eventSize(size),
                eventMask(size-1),
                bufferHead(0),
                bufferTail(0){
            assert((size!=0) && ((size&(~size+1))==size));
        }
        ~SimpleEventQueueBuffer(){
            if(eventBuffer){
                delete [] eventBuffer;
            }
        }
        int32_t EnqueueEventBuffer(GSXR_Event* event){
            const size_t head = bufferHead.load(std::memory_order_relaxed);
            if (((bufferTail.load(std::memory_order_acquire) - (head + 1)) & eventMask) >= 1) {
                eventBuffer[head & eventMask] = *event;
                bufferHead.store(head + 1, std::memory_order_release);
                return 0;
            }
            return -1;
        }
        int32_t DequeueEventBuffer(GSXR_Event* event){
            const size_t tail = bufferTail.load(std::memory_order_relaxed);
            if (((bufferHead.load(std::memory_order_acquire) - tail) & eventMask) >= 1) {
                *event = eventBuffer[bufferTail & eventMask];
                bufferTail.store(tail + 1, std::memory_order_release);
                return 0;
            }
            return -1;
        }
    }SimpleEventQueueBuffer;
    typedef struct SimpleEventQueue{
        SimpleEventQueueBuffer* eventQueueBuffer;
        timespec timebase;
        pid_t threadId;
        SimpleEventQueue(size_t size,timespec time):
                eventQueueBuffer(new SimpleEventQueueBuffer(size)),
                threadId(gettid()),
                timebase(time){}
        ~SimpleEventQueue(){
            if(eventQueueBuffer){
                delete eventQueueBuffer;
            }
            threadId=-1;
        }
    }SimpleEventQueue;
    class SimpleEventManager {
    public:
        static SimpleEventManager* GetEventManager();
        ~SimpleEventManager();
        int32_t SubmitEvent(GSXR_Event*);
        int32_t PollAppEvent(GSXR_Event*);
        void PollAllEvents();
    private:
        SimpleEventManager();
    private:
        SimpleEventQueue* appEventQueue;
        SimpleEventQueue* rtEventQueue[EVENT_QUEUES];
        timespec timeBase;
    };
}



#endif //GSXRRUNTIME_SIMPLEEVENTMANAGER_H
