package com.gsxr.simpleruntime;

import android.accessibilityservice.FingerprintGestureController;
import android.annotation.SuppressLint;
import android.app.Activity;
import android.opengl.GLSurfaceView;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;

import java.util.ArrayList;

import in.championswimmer.sfg.lib.SimpleFingerGestures;

/**
 *  Created     :   2021-03-1
 *  Copyright (c)  CMCC Technologies, All rights reserved.
 */


public class GsxrJavaApi {

    private static GsxrJavaApi gsxrJavaApi;
    private final String TAG="GsxrJavaApi";

    public native void touchUpCallBack();
    public native void touchDownCallBack();
    public native void touchLeftCallBack();
    public native void touchRightCallBack();
    public native void singleTapCallBack();
    public native void updateTouchDataCallBack(float[] outData);
    public native void SingleTouchDownEventCallBack(boolean outData);

    public static GsxrJavaApi GetGsxrApiInstance(){
        if(gsxrJavaApi==null){
            gsxrJavaApi = new GsxrJavaApi();
        }
        return gsxrJavaApi;
    }

   public GsxrJavaApi(){
       Log.d(TAG,"GsxrJavaApi");
   }

   @SuppressLint("ClickableViewAccessibility")
   public void RegisterViewTouch(Activity activity){
       Log.d(TAG,"RegisterViewTouch activity: "+activity);
       View view = activity.findViewById(android.R.id.content);
       ArrayList<View> viewArrayList = new ArrayList<View>();
       {
           if(view instanceof ViewGroup){
               ViewGroup viewGroup = (ViewGroup) view;
               for(int i=0;i<viewGroup.getChildCount();i++){
                   View child = viewGroup.getChildAt(i);
                   viewArrayList.add(child);
                   Log.d(TAG,"Get child view:"+i+ " type: "+child.getClass().getName());
                   Log.d(TAG,"Get view:"+i+" size: "+child.getWidth()+" "+child.getHeight());
               }
           }
       }
       if(viewArrayList.size()==0){
           Log.d(TAG,"no view!");
           return;
       }
       SimpleFingerGestures simpleFingerGestures = new SimpleFingerGestures();
//       view.setOnTouchListener(simpleFingerGestures);
       viewArrayList.get(0).setOnTouchListener(simpleFingerGestures);
       simpleFingerGestures.setDebug(true);
       simpleFingerGestures.setConsumeTouchEvents(true);
       simpleFingerGestures.setOnFingerGestureListener(new SimpleFingerGestures.OnFingerGestureListener() {
           @Override
           public boolean onSwipeUp(int fingers, long gestureDuration, double gestureDistance) {
               Log.d(TAG,"onSwipeUp");
               touchUpCallBack();
               return false;
           }

           @Override
           public boolean onSwipeDown(int fingers, long gestureDuration, double gestureDistance) {
               Log.d(TAG,"onSwipeDown");
               touchDownCallBack();
               return false;
           }

           @Override
           public boolean onSwipeLeft(int fingers, long gestureDuration, double gestureDistance) {
               Log.d(TAG,"onSwipeLeft");
               touchLeftCallBack();
               return false;
           }

           @Override
           public boolean onSwipeRight(int fingers, long gestureDuration, double gestureDistance) {
               Log.d(TAG,"onSwipeRight");
               touchRightCallBack();
               return false;
           }

           @Override
           public boolean onPinch(int fingers, long gestureDuration, double gestureDistance) {
               Log.d(TAG,"onPinch");
               return false;
           }

           @Override
           public boolean onUnpinch(int fingers, long gestureDuration, double gestureDistance) {
               Log.d(TAG,"onUnpinch");
               return false;
           }

           @Override
           public boolean onDoubleTap(int fingers) {
               Log.d(TAG,"onDoubleTap");
               return false;
           }
           @Override
           public boolean onSingleTap(int fingers) {
               Log.d(TAG,"onSingleTap");
               singleTapCallBack();
               return false;
           }
           @Override
           public boolean onUpdateTouchData(double[] data) {
               float[] inData={(float) data[0], (float) data[1], (float) data[2], (float) data[3]};
               Log.d(TAG,"onUpdateTouchData "+inData[0]+" "+inData[1]+" "+inData[2]+" "+inData[3]);
               updateTouchDataCallBack(inData);
               return false;
           }
           @Override
           public boolean onSingleTapUp() {
               SingleTouchDownEventCallBack(false);
               return false;
           }

           @Override
           public boolean onSingleTapDown() {
               SingleTouchDownEventCallBack(true);
               return false;
           }
       });
   }

}
